@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2008-2013 Vaclav Slavik
+ *  Copyright (C) 2008-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -38,10 +38,6 @@
 #include "osx_helpers.h"
 #endif
 
-#if defined(__WXMSW__) && wxCHECK_VERSION(2,9,0)
-    #define USE_SLIDE_EFFECT
-#endif
-
 #ifdef __WXMAC__
     #define SMALL_BORDER   5
     #define BUTTONS_SPACE 10
@@ -59,7 +55,11 @@ AttentionBar::AttentionBar(wxWindow *parent)
     : wxPanel(parent, wxID_ANY, wxDefaultPosition, wxDefaultSize,
               wxTAB_TRAVERSAL | wxBORDER_NONE)
 {
+#ifdef __WXMSW__
+    wxColour bg("#FFF499"); // match Visual Studio 2012+'s aesthetics
+#else
     wxColour bg = wxSystemSettings::GetColour(wxSYS_COLOUR_INFOBK);
+#endif
     SetBackgroundColour(bg);
     SetForegroundColour(wxSystemSettings::GetColour(wxSYS_COLOUR_INFOTEXT));
 
@@ -72,7 +72,7 @@ AttentionBar::AttentionBar(wxWindow *parent)
             new wxBitmapButton
                 (
                     this, wxID_CLOSE,
-                    wxArtProvider::GetBitmap(_T("window-close"), wxART_MENU),
+                    wxArtProvider::GetBitmap("window-close", wxART_MENU),
                     wxDefaultPosition, wxDefaultSize,
                     wxNO_BORDER
                 );
@@ -80,15 +80,15 @@ AttentionBar::AttentionBar(wxWindow *parent)
 #ifdef __WXMSW__
     btnClose->SetBackgroundColour(bg);
 #endif
-
 #ifdef __WXMAC__
     SetWindowVariant(wxWINDOW_VARIANT_SMALL);
+#endif
+#if defined(__WXMAC__) || defined(__WXMSW__)
     Bind(wxEVT_PAINT, &AttentionBar::OnPaint, this);
-
     wxFont boldFont = wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
     boldFont.SetWeight(wxFONTWEIGHT_BOLD);
     m_label->SetFont(boldFont);
-#endif // __WXMAC__
+#endif //
 
     wxSizer *sizer = new wxBoxSizer(wxHORIZONTAL);
     sizer->AddSpacer(wxSizerFlags::GetDefaultBorder());
@@ -127,6 +127,18 @@ void AttentionBar::OnPaint(wxPaintEvent&)
 }
 #endif // __WXMAC__
 
+#ifdef __WXMSW__
+void AttentionBar::OnPaint(wxPaintEvent&)
+{
+    wxPaintDC dc(this);
+
+    wxRect rect(dc.GetSize());
+    dc.SetBrush(*wxTRANSPARENT_BRUSH);
+    dc.SetPen(wxColour("#e5dd8c"));
+    dc.DrawRoundedRectangle(rect, 1);
+}
+#endif // __WXMSW__
+
 
 void AttentionBar::ShowMessage(const AttentionMessage& msg)
 {
@@ -148,12 +160,8 @@ void AttentionBar::ShowMessage(const AttentionMessage& msg)
             break;
     }
 
-    m_icon->SetBitmap(wxArtProvider::GetBitmap(iconName, wxART_MENU));
-#if wxCHECK_VERSION(2,9,0)
+    m_icon->SetBitmap(wxArtProvider::GetBitmap(iconName, wxART_MENU, wxSize(16, 16)));
     m_label->SetLabelText(msg.m_text);
-#else
-    m_label->SetLabel(msg.m_text);
-#endif
 
     m_buttons->Clear(true/*delete_windows*/);
     m_actions.clear();
@@ -175,21 +183,13 @@ void AttentionBar::ShowMessage(const AttentionMessage& msg)
             GetBestSize().y);
     Layout();
 
-#ifdef USE_SLIDE_EFFECT
-    ShowWithEffect(wxSHOW_EFFECT_SLIDE_TO_BOTTOM);
-#else
     Show();
-#endif
     GetParent()->Layout();
 }
 
 void AttentionBar::HideMessage()
 {
-#ifdef USE_SLIDE_EFFECT
-    HideWithEffect(wxSHOW_EFFECT_SLIDE_TO_TOP);
-#else
     Hide();
-#endif
     GetParent()->Layout();
 }
 
@@ -207,8 +207,11 @@ void AttentionBar::OnAction(wxCommandEvent& event)
         return;
     }
 
-    HideMessage();
+    // first perform the action...
     i->second();
+
+    // ...then hide the message
+    HideMessage();
 }
 
 
@@ -217,7 +220,7 @@ void AttentionMessage::AddToBlacklist(const wxString& id)
 {
     wxConfig::Get()->Write
         (
-            wxString::Format(_T("/messages/dont_show/%s"), id.c_str()),
+            wxString::Format("/messages/dont_show/%s", id.c_str()),
             (long)true
         );
 }
@@ -225,15 +228,15 @@ void AttentionMessage::AddToBlacklist(const wxString& id)
 /* static */
 bool AttentionMessage::IsBlacklisted(const wxString& id)
 {
-    return wxConfig::Get()->Read
+    return wxConfig::Get()->ReadBool
         (
-            wxString::Format(_T("/messages/dont_show/%s"), id.c_str()),
-            (long)false
+            wxString::Format("/messages/dont_show/%s", id.c_str()),
+            false
         );
 }
 
 void AttentionMessage::AddDontShowAgain()
 {
     AddAction(_("Don't show again"),
-              boost::bind(&AttentionMessage::AddToBlacklist, m_id));
+              std::bind(&AttentionMessage::AddToBlacklist, m_id));
 }
