@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2008-2013 Vaclav Slavik
+ *  Copyright (C) 2008-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -27,10 +27,10 @@
 #define _ATTENTIONBAR_H_
 
 #include <wx/panel.h>
+
 #include <vector>
 #include <map>
-#include <boost/function.hpp>
-#include <boost/bind.hpp>
+#include <functional>
 
 class WXDLLIMPEXP_FWD_CORE wxStaticText;
 class WXDLLIMPEXP_FWD_CORE wxStaticBitmap;
@@ -63,7 +63,7 @@ public:
     AttentionMessage(const wxString& id, Kind kind, const wxString& text)
         : m_id(id), m_kind(kind), m_text(text) {}
 
-    typedef boost::function<void()> Callback;
+    typedef std::function<void()> Callback;
 
     /**
         Adds an action button to the bar. By default, a close button is shown,
@@ -121,7 +121,7 @@ public:
 private:
     void OnClose(wxCommandEvent& event);
     void OnAction(wxCommandEvent& event);
-#ifdef __WXMAC__
+#if defined(__WXMAC__) || defined(__WXMSW__)
     void OnPaint(wxPaintEvent& event);
 #endif
 
