@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 1999-2013 Vaclav Slavik
+ *  Copyright (C) 1999-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -26,6 +26,8 @@
 #ifndef _CATALOG_H_
 #define _CATALOG_H_
 
+#include "language.h"
+
 #include <wx/encconv.h>
 #include <wx/regex.h>
 #include <wx/arrstr.h>
@@ -64,10 +66,8 @@ class CatalogItem
 {
     public:
         /// Ctor. Initializes the object with source string and translation.
-        CatalogItem(const wxString& str = wxEmptyString,
-                    const wxString& str_plural = wxEmptyString)
-                : m_string(str),
-                  m_plural(str_plural),
+        CatalogItem()
+                : m_id(0),
                   m_hasPlural(false),
                   m_hasContext(false),
                   m_references(),
@@ -80,7 +80,8 @@ class CatalogItem
                   m_bookmark(NO_BOOKMARK) {}
 
         CatalogItem(const CatalogItem& dt)
-                : m_string(dt.m_string),
+                : m_id(dt.m_id),
+                  m_string(dt.m_string),
                   m_plural(dt.m_plural),
                   m_hasPlural(dt.m_hasPlural),
                   m_hasContext(dt.m_hasContext),
@@ -101,6 +102,9 @@ class CatalogItem
                   m_errorString(dt.m_errorString),
                   m_bookmark(dt.m_bookmark) {}
 
+        /// Gets numeric, 1-based ID
+        int GetId() const { return m_id; }
+
         /// Returns the source string.
         const wxString& GetString() const { return m_string; }
 
@@ -119,7 +123,7 @@ class CatalogItem
 
         /// How many translations (plural forms) do we have?
         unsigned GetNumberOfTranslations() const
-            { return m_translations.size(); }
+            { return (unsigned)m_translations.size(); }
 
         /// Returns number of plural forms in this translation; note that this
         /// may be less than what the header says, because some may be
@@ -149,6 +153,9 @@ class CatalogItem
         /// Convenience function: does this entry has a comment?
         bool HasComment() const { return !m_comment.empty(); }
 
+        /// Convenience function: does this entry has auto comments?
+        bool HasAutoComments() const { return !m_autocomments.empty(); }
+
         /// Adds new reference to the entry (used by SourceDigger).
         void AddReference(const wxString& ref)
         {
@@ -156,6 +163,8 @@ class CatalogItem
                 m_references.Add(ref);
         }
 
+        void SetId(int id) { m_id = id; }
+
         /// Sets the string.
         void SetString(const wxString& s)
         {
@@ -201,7 +210,7 @@ class CatalogItem
         wxString GetFlags() const;
 
         /// Sets fuzzy flag.
-        void SetFuzzy(bool fuzzy) { m_isFuzzy = fuzzy; }
+        void SetFuzzy(bool fuzzy);
         /// Gets value of fuzzy flag.
         bool IsFuzzy() const { return m_isFuzzy; }
         /// Sets translated flag.
@@ -251,8 +260,7 @@ class CatalogItem
         /** Checks if %i etc. are correct in the translation (true if yes).
             Strings that are not c-format are always correct. */
         Validity GetValidity() const { return m_validity; }
-        void SetValidity(bool val)
-            { m_validity = val ? Val_Valid : Val_Invalid; }
+        void SetValidity(Validity val) { m_validity = val; }
 
         void SetErrorString(const wxString& str) { m_errorString = str; }
         wxString GetErrorString() const { return m_errorString; }
@@ -265,6 +273,8 @@ class CatalogItem
         void SetBookmark(Bookmark bookmark) {m_bookmark = bookmark;}
 
     private:
+        int m_id;
+
         wxString m_string, m_plural;
         bool m_hasPlural;
 
@@ -437,11 +447,12 @@ class Catalog
 
             // Parsed values:
 
-            wxString LanguageCode, Project, CreationDate,
+            wxString Project, CreationDate,
                      RevisionDate, Translator, TranslatorEmail,
                      Team, TeamEmail, Charset, SourceCodeCharset;
+            Language Lang;
 
-            wxArrayString SearchPaths, Keywords;
+            wxArrayString SearchPaths, SearchPathsExcluded, Keywords;
             int Bookmarks[BOOKMARK_LAST];
             wxString BasePath;
 
@@ -455,7 +466,15 @@ class Catalog
 
         enum CreationFlags
         {
-            CreationFlag_IgnoreHeader = 1
+            CreationFlag_IgnoreHeader       = 1,
+            CreationFlag_IgnoreTranslations = 2
+        };
+
+        enum class CompilationStatus
+        {
+            NotDone,
+            Success,
+            Error
         };
 
         /// Default ctor. Creates empty catalog, you have to call Load.
@@ -463,7 +482,7 @@ class Catalog
 
         /// Ctor that loads the catalog from \a po_file with Load.
         /// \a flags is CreationFlags combination.
-        Catalog(const wxString& po_file, int flags = 0);
+        explicit Catalog(const wxString& po_file, int flags = 0);
 
         ~Catalog();
 
@@ -498,7 +517,9 @@ class Catalog
             Note that \a po_file refers to .po file, .mo file will have same
             name & location as .po file except for different extension.
          */
-        bool Save(const wxString& po_file, bool save_mo, int& validation_errors);
+        bool Save(const wxString& po_file, bool save_mo,
+                  int& validation_errors,
+                  CompilationStatus& mo_compilation_status);
 
         /// Exports the catalog to HTML format
         bool ExportToHTML(const wxString& filename);
@@ -506,17 +527,21 @@ class Catalog
         /** Updates the catalog from sources.
             \see SourceDigger, Parser, UpdateFromPOT.
          */
-        bool Update(ProgressInfo *progress, bool summary = true);
+        bool Update(ProgressInfo *progress, bool summary, bool *cancelledByUser);
 
         /** Updates the catalog from POT file.
             \see Update
          */
         bool UpdateFromPOT(const wxString& pot_file,
-                           bool summary = true,
+                           bool summary,
+                           bool *cancelledByUser,
                            bool replace_header = false);
 
         /// Returns the number of strings/translations in the catalog.
-        size_t GetCount() const { return m_items.size(); }
+        unsigned GetCount() const { return (unsigned)m_items.size(); }
+
+        /// Is the catalog empty?
+        bool empty() const { return m_items.empty(); }
 
         /** Returns number of all, fuzzy, badtokens and untranslated items.
             Any argument may be NULL if the caller is not interested in
@@ -554,10 +579,8 @@ class Catalog
          */
         bool IsOk() const { return m_isOk; }
 
-        /** Returns xx_YY ISO code of catalog's language if either the Poedit
-            extensions headers are present or if filename is known and is in
-            the xx[_YY] form, otherwise returns empty string. */
-        wxString GetLocaleCode() const;
+        /** Returns catalog's language (may be invalid). */
+        Language GetLanguage() const { return m_header.Lang; }
 
         /// Adds entry to the catalog (the catalog will take ownership of
         /// the object).
@@ -591,7 +614,13 @@ class Catalog
         /// Returns number of errors (i.e. 0 if no errors).
         int Validate();
 
+        const wxString& GetFileName() const { return m_fileName; }
+        void SetFileName(const wxString& fn) { m_fileName = fn; }
+
     protected:
+        /// Fix commonly encountered fixable problems with loaded files
+        void FixupCommonIssues();
+
         int DoValidate(const wxString& po_file);
         bool DoSaveOnly(const wxString& po_file);
         bool DoSaveOnly(wxTextFile& f, wxTextFileType crlf);
@@ -620,7 +649,7 @@ class Catalog
 
             \return true if the merge was OK'ed by the user, false otherwise
          */
-        bool ShowMergeSummary(Catalog *refcat);
+        bool ShowMergeSummary(Catalog *refcat, bool *cancelledByUser);
 
     private:
         CatalogItemArray m_items;
@@ -640,7 +669,8 @@ class CatalogParser
     public:
         CatalogParser(wxTextFile *f)
             : m_textFile(f),
-              m_ignoreHeader(false)
+              m_ignoreHeader(false),
+              m_ignoreTranslations(false)
         {}
 
         virtual ~CatalogParser() {}
@@ -648,6 +678,9 @@ class CatalogParser
         /// Tell the parser to ignore header entries when processing
         void IgnoreHeader(bool ignore) { m_ignoreHeader = ignore; }
 
+        /// Tell the parser to treat input as POT and ignore translations
+        void IgnoreTranslations(bool ignore) { m_ignoreTranslations = ignore; }
+
         /** Parses the entire file, calls OnEntry each time
             new msgid/msgstr pair is found.
 
@@ -687,11 +720,16 @@ class CatalogParser
             return true;
         }
 
+        virtual void OnIgnoredEntry() {}
+
         /// Textfile being parsed.
         wxTextFile *m_textFile;
 
         /// Whether the header should be parsed or not
         bool m_ignoreHeader;
+
+        /// Whether the translations should be ignored (as if it was a POT)
+        bool m_ignoreTranslations;
 };
 
 #endif // _CATALOG_H_
