@@ -1,7 +1,7 @@
-/*
- *  This file is part of Poedit (http://www.poedit.net)
+﻿/*
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2003-2013 Vaclav Slavik
+ *  Copyright (C) 2003-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -27,23 +27,24 @@
 
 #if NEED_CHOOSELANG_UI
 
+#include "language.h"
+
 #include <wx/wx.h>
 #include <wx/config.h>
-#include <wx/intl.h>
 #include <wx/translation.h>
 
 static void SaveUILanguage(const wxString& lang)
 {
     if (lang.empty())
-        wxConfig::Get()->Write(_T("ui_language"), _T("default"));
+        wxConfig::Get()->Write("ui_language", "default");
     else
-        wxConfig::Get()->Write(_T("ui_language"), lang);
+        wxConfig::Get()->Write("ui_language", lang);
 }
 
 wxString GetUILanguage()
 {
-    wxString lng = wxConfig::Get()->Read(_T("ui_language"));
-    if (!lng.empty() && lng != _T("default"))
+    wxString lng = wxConfig::Get()->Read("ui_language");
+    if (!lng.empty() && lng != "default")
         return lng;
     else
         return "";
@@ -56,13 +57,13 @@ static bool ChooseLanguage(wxString *value)
 
     {
         wxBusyCursor bcur;
-        langs = wxTranslations::Get()->GetAvailableTranslations(_T("poedit"));
+        langs = wxTranslations::Get()->GetAvailableTranslations("poedit");
 
         arr.push_back(_("(Use default language)"));
-        for ( wxArrayString::const_iterator i = langs.begin(); i != langs.end(); ++i )
+        for (auto i : langs)
         {
-            const wxLanguageInfo *langInfo = wxLocale::FindLanguageInfo(*i);
-            arr.push_back(langInfo ? langInfo->Description : *i);
+            auto lang = Language::TryParse(i);
+            arr.push_back(lang.DisplayNameInItself() + L"  —  " + lang.DisplayName());
         }
     }
     int choice = wxGetSingleChoiceIndex(
@@ -86,7 +87,7 @@ void ChangeUILanguage()
         return;
     SaveUILanguage(lang);
     wxMessageBox(_("You must restart Poedit for this change to take effect."),
-                 _T("Poedit"),
+                 "Poedit",
                  wxOK | wxCENTRE | wxICON_INFORMATION);
 }
 
