@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2003-2013 Vaclav Slavik
+ *  Copyright (C) 2003-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -29,19 +29,10 @@
 #include <wx/string.h>
 #include <wx/intl.h>
 
-#if defined(__UNIX__) && !defined(__WXMAC__)
-    #define NEED_CHOOSELANG_UI 0
+#ifdef __WXMSW__
+    #define NEED_CHOOSELANG_UI 1
 #else
-    #if wxCHECK_VERSION(2,9,1)
-        #define NEED_CHOOSELANG_UI 1
-    #else
-        #ifdef __VISUALC__
-            #pragma message ("disabling language-choosing UI for wxWidgets < 2.9.1")
-        #else
-            #warning "disabling language-choosing UI for wxWidgets < 2.9.1"
-        #endif
-        #define NEED_CHOOSELANG_UI 0
-    #endif
+    #define NEED_CHOOSELANG_UI 0
 #endif
 
 #if NEED_CHOOSELANG_UI
