@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2001-2013 Vaclav Slavik
+ *  Copyright (C) 2001-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -34,7 +34,7 @@
 
 CommentDialog::CommentDialog(wxWindow *parent, const wxString& comment) : wxDialog()
 {
-    wxXmlResource::Get()->LoadDialog(this, parent, _T("comment_dlg"));
+    wxXmlResource::Get()->LoadDialog(this, parent, "comment_dlg");
     m_text = XRCCTRL(*this, "comment", wxTextCtrl);
 
     m_text->SetValue(RemoveStartHash(comment));
@@ -59,17 +59,17 @@ void CommentDialog::OnClear(wxCommandEvent&)
 /*static*/ wxString CommentDialog::RemoveStartHash(const wxString& comment)
 {
     wxString tmpComment;
-    wxStringTokenizer tkn(comment, _T("\n\r"));
+    wxStringTokenizer tkn(comment, "\n\r");
     while (tkn.HasMoreTokens())
-        tmpComment << tkn.GetNextToken().Mid(2) << _T("\n");
+        tmpComment << tkn.GetNextToken().Mid(2) << "\n";
     return tmpComment;
 }
 
 /*static*/ wxString CommentDialog::AddStartHash(const wxString& comment)
 {
     wxString tmpComment;
-    wxStringTokenizer tkn(comment, _T("\n\r"));
+    wxStringTokenizer tkn(comment, "\n\r");
     while (tkn.HasMoreTokens())
-        tmpComment << _T("# ") << tkn.GetNextToken() << _T("\n");
+        tmpComment << "# " << tkn.GetNextToken() << "\n";
     return tmpComment;
 }
