@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2000-2013 Vaclav Slavik
+ *  Copyright (C) 2000-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -46,12 +46,12 @@ bool ConcatCatalogs(const wxArrayString& files, const wxString& outfile)
     for ( wxArrayString::const_iterator i = files.begin();
           i != files.end(); ++i )
     {
-        list += wxString::Format(_T(" \"%s\""), i->c_str());
+        list += wxString::Format(" %s", QuoteCmdlineArg(*i));
     }
 
     wxString cmd =
-        wxString::Format(_T("msgcat --force-po -o \"%s\" %s"),
-                         outfile.c_str(),
+        wxString::Format("msgcat --force-po -o %s %s",
+                         QuoteCmdlineArg(outfile),
                          list.c_str());
     bool succ = ExecuteGettext(cmd);
 
@@ -65,9 +65,25 @@ bool ConcatCatalogs(const wxArrayString& files, const wxString& outfile)
     return true;
 }
 
+// Returns true if the filename is in 'paths' prefixes list
+bool FilenameMatchesPathsList(const wxString& fn, const wxArrayString& paths)
+{
+    for (auto p: paths)
+    {
+        if (fn == p)
+            return true;
+        if (fn.StartsWith(p + "/"))
+            return true;
+    }
+
+    return false;
+}
+
+
 } // anonymous namespace
 
 Catalog *SourceDigger::Dig(const wxArrayString& paths,
+                           const wxArrayString& excludePaths,
                            const wxArrayString& keywords,
                            const wxString& charset)
 {
@@ -76,7 +92,7 @@ Catalog *SourceDigger::Dig(const wxArrayString& paths,
 
     m_progressInfo->UpdateMessage(_("Scanning files..."));
 
-    wxArrayString *all_files = FindFiles(paths, pdb);
+    wxArrayString *all_files = FindFiles(paths, excludePaths, pdb);
     if (all_files == NULL)
         return NULL;
 
@@ -89,6 +105,7 @@ Catalog *SourceDigger::Dig(const wxArrayString& paths,
             continue; // no files of this kind
 
         m_progressInfo->UpdateMessage(
+            // TRANSLATORS: '%s' is replaced with the kind of the files (e.g. C++, PHP, ...)
             wxString::Format(_("Parsing %s files..."), pdb[i].Name.c_str()));
         if (!DigFiles(tmpdir, partials, all_files[i], pdb[i], keywords, charset))
         {
@@ -102,7 +119,7 @@ Catalog *SourceDigger::Dig(const wxArrayString& paths,
     if ( partials.empty() )
         return NULL; // couldn't parse any source files
 
-    wxString mergedFile = tmpdir.CreateFileName(_T("merged.pot"));
+    wxString mergedFile = tmpdir.CreateFileName("merged.pot");
 
     if ( !ConcatCatalogs(partials, mergedFile) )
         return NULL;
@@ -142,7 +159,7 @@ bool SourceDigger::DigFiles(TempDirectory& tmpdir,
             batchfiles.Add(files[i]);
         last = i;
 
-        wxString tempfile = tmpdir.CreateFileName(_T("extracted.pot"));
+        wxString tempfile = tmpdir.CreateFileName("extracted.pot");
         if (!ExecuteGettext(
                     parser.GetCommand(batchfiles, keywords, tempfile, charset)))
         {
@@ -151,7 +168,7 @@ bool SourceDigger::DigFiles(TempDirectory& tmpdir,
 
         tempfiles.push_back(tempfile);
 
-        m_progressInfo->UpdateGauge(batchfiles.GetCount());
+        m_progressInfo->UpdateGauge((int)batchfiles.GetCount());
 
         if (m_progressInfo->Cancelled())
             return false;
@@ -160,7 +177,7 @@ bool SourceDigger::DigFiles(TempDirectory& tmpdir,
     if ( tempfiles.empty() )
         return false; // failed to parse any source files
 
-    wxString outfile = tmpdir.CreateFileName(_T("merged_chunks.pot"));
+    wxString outfile = tmpdir.CreateFileName("merged_chunks.pot");
     if ( !ConcatCatalogs(tempfiles, outfile) )
         return false;
 
@@ -170,7 +187,8 @@ bool SourceDigger::DigFiles(TempDirectory& tmpdir,
 
 
 
-wxArrayString *SourceDigger::FindFiles(const wxArrayString& paths, 
+wxArrayString *SourceDigger::FindFiles(const wxArrayString& paths,
+                                       const wxArrayString& excludePaths,
                                        ParsersDB& pdb)
 {
     if (pdb.GetCount() == 0)
@@ -181,28 +199,42 @@ wxArrayString *SourceDigger::FindFiles(const wxArrayString& paths,
     
     for (i = 0; i < paths.GetCount(); i++)
     {
-        if ( !FindInDir(paths[i], files) )
+        if ( !FindInDir(paths[i], excludePaths, files) )
+        {
             wxLogWarning(_("No files found in: ") + paths[i]);
+        }
     }
 
+    // Sort the filenames in some well-defined order. This is because directory
+    // traversal has, generally speaking, undefined order, and the order differs
+    // between filesystems. Finally, the order is reflected in the created PO
+    // files and it is much better for diffs if it remains consistent.
+    files.Sort();
+
     size_t filescnt = 0;
     for (i = 0; i < pdb.GetCount(); i++)
     {
         p_files[i] = pdb[i].SelectParsable(files);
         filescnt += p_files[i].GetCount();
     }
-    m_progressInfo->SetGaugeMax(filescnt);
+    m_progressInfo->SetGaugeMax((int)filescnt);
     
     if (filescnt == 0)
+    {
         wxLogError(_("Poedit did not find any files in scanned directories."));
+    }
 
     return p_files;
 }
 
 
 
-int SourceDigger::FindInDir(const wxString& dirname, wxArrayString& files)
+int SourceDigger::FindInDir(const wxString& dirname,
+                            const wxArrayString& excludePaths,
+                            wxArrayString& files)
 {
+    if (dirname.empty())
+        return 0;
     wxDir dir(dirname);
     if (!dir.IsOpened()) 
         return 0;
@@ -213,16 +245,26 @@ int SourceDigger::FindInDir(const wxString& dirname, wxArrayString& files)
     cont = dir.GetFirst(&filename, wxEmptyString, wxDIR_FILES);
     while (cont)
     {
-        files.Add(dirname + _T("/") + filename);
-        found++;
+        const wxString f = (dirname == ".") ? filename : dirname + "/" + filename;
         cont = dir.GetNext(&filename);
-    }    
+
+        if (FilenameMatchesPathsList(f, excludePaths))
+            continue;
+
+        files.Add(f);
+        found++;
+    }
 
     cont = dir.GetFirst(&filename, wxEmptyString, wxDIR_DIRS);
     while (cont)
     {
-        found += FindInDir(dirname + _T("/") + filename, files);
+        const wxString f = (dirname == ".") ? filename : dirname + "/" + filename;
         cont = dir.GetNext(&filename);
+
+        if (FilenameMatchesPathsList(f, excludePaths))
+            continue;
+
+        found += FindInDir(f, excludePaths, files);
     }
 
     return found;
