@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2000-2013 Vaclav Slavik
+ *  Copyright (C) 2000-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -45,7 +45,7 @@ class SourceDigger
 {
     public:
         /// Ctor. \a pi is used to display the progress of parsing.
-        SourceDigger(ProgressInfo *pi) : m_progressInfo(pi) {}
+        SourceDigger(ProgressInfo *progress) : m_progressInfo(progress) {}
 
         /** Scans files for translatable strings and returns Catalog
             instance containing them. All files in input \a paths that 
@@ -54,11 +54,13 @@ class SourceDigger
             program (typically, gettext) according to parser definition.
 
             \param paths    list of directories to look in
+            \param excludePaths directories to skip
             \param keywords list of keywords that are recognized as 
                             prefixes for translatable strings in sources
             \param charset  source code charset (may be empty)
          */
-        Catalog *Dig(const wxArrayString& paths, 
+        Catalog *Dig(const wxArrayString& paths,
+                     const wxArrayString& excludePaths,
                      const wxArrayString& keywords,
                      const wxString& charset);
 
@@ -68,12 +70,16 @@ class SourceDigger
             array holds list of files that can be parsed by n-th parser
             in \a pdb database.
          */
-        wxArrayString *FindFiles(const wxArrayString& paths, ParsersDB& pdb);
+        wxArrayString *FindFiles(const wxArrayString& paths,
+                                 const wxArrayString& excludePaths,
+                                 ParsersDB& pdb);
 
         /** Finds all files in given directory.
             \return false if an error occured.
          */
-        int FindInDir(const wxString& dirname, wxArrayString& files);
+        int FindInDir(const wxString& dirname,
+                      const wxArrayString& excludePaths,
+                      wxArrayString& files);
 
         /** Digs translatable strings from given files.
             \param outfiles list to which (temporary) file name of extracted
