@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 1999-2013 Vaclav Slavik
+ *  Copyright (C) 1999-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -33,45 +33,60 @@
 #include <wx/docview.h>
 
 class WXDLLIMPEXP_FWD_BASE wxConfigBase;
+class WXDLLIMPEXP_FWD_CORE wxMenuBar;
 
 
 /// wxApp for use with 
 class PoeditApp : public wxApp
 {
     public:
+        PoeditApp();
+        ~PoeditApp();
+
         /** wxWin initalization hook. Shows PoeditFrame and initializes
             configuration entries to default values if they were missing.
          */
         virtual bool OnInit();
         virtual int OnExit();
 
-        /** Gets application's path. This path is used when looking for 
-            resources.zip and help files, both of them can be found
-            in {appPath}/share/poedit. Looks in registry under Windows
-            and returns value of POEDIT_PREFIX which is supplied at
-            compilation time and equals configure's --prefix argument
-            (unless $POEDIT_PREFIX environment variable exists, in which
-            case its content is returned).
-         */
-        wxString GetAppPath() const;
+        virtual wxLayoutDirection GetLayoutDirection() const;
 
         /// Returns Poedit version string.
         wxString GetAppVersion() const;
+        bool IsBetaVersion() const;
+        bool CheckForBetaUpdates() const;
 
-        // opens a file in new frame
-        void OpenFile(const wxString& name);
+        // opens files in new frame
+        void OpenFiles(const wxArrayString& filenames);
         // opens empty frame or catalogs manager
         void OpenNewFile();
 
+#ifndef __WXOSX__
         wxFileHistory& FileHistory() { return m_history; }
+#endif
 
 #ifdef __WXMAC__
-        virtual void MacOpenFile(const wxString& name) { OpenFile(name); }
+        virtual void MacOpenFiles(const wxArrayString& names) { OpenFiles(names); }
         virtual void MacNewFile() { OpenNewFile(); }
 #endif
 
         void EditPreferences();
 
+        virtual bool OnExceptionInMainLoop();
+
+        // Open page on poedit.net in the browser
+        void OpenPoeditWeb(const wxString& path);
+
+#ifdef __WXOSX__
+        // Make OSX-specific modifications to the menus, e.g. adding items into
+        // the apple menu etc. Call on every newly created menubar
+        void TweakOSXMenuBar(wxMenuBar *bar);
+        void CreateFakeOpenRecentMenu();
+        void InstallOpenRecentMenu(wxMenuBar *bar);
+        void OnIdleInstallOpenRecentMenu(wxIdleEvent& event);
+        virtual void OSXOnWillFinishLaunching();
+#endif
+
     protected:
         /** Sets default values to configuration items that don't
             have anything set. (This may happen after fresh installation or
@@ -85,23 +100,34 @@ class PoeditApp : public wxApp
         
     private:
         void SetupLanguage();
-        void AskForDonations(wxWindow *parent);
 
         // App-global menu commands:
         void OnNew(wxCommandEvent& event);
         void OnOpen(wxCommandEvent& event);
+#ifndef __WXOSX__
         void OnOpenHist(wxCommandEvent& event);
+#endif
         void OnAbout(wxCommandEvent& event);
         void OnManager(wxCommandEvent& event);
         void OnQuit(wxCommandEvent& event);
         void OnPreferences(wxCommandEvent& event);
         void OnHelp(wxCommandEvent& event);
+        void OnGettextManual(wxCommandEvent& event);
+
+#ifdef __WXMSW__
+        void AssociateFileTypeIfNeeded();
+        void OnWinsparkleCheck(wxCommandEvent& event);
+        static int WinSparkle_CanShutdown();
+        static void WinSparkle_Shutdown();
+#endif
 
         DECLARE_EVENT_TABLE()
 
+#ifdef __WXOSX__
+        class RecentMenuData;
+        std::unique_ptr<RecentMenuData> m_recentMenuData;
+#else
         wxFileHistory m_history;
-#if !wxCHECK_VERSION(2,9,1)
-        wxLocale m_locale;
 #endif
 };
 
