@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 1999-2013 Vaclav Slavik
+ *  Copyright (C) 1999-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -26,14 +26,12 @@
 #ifndef _EDFRAME_H_
 #define _EDFRAME_H_
 
-#if defined(__WXMSW__)
-  #define CAN_MODIFY_DEFAULT_FONT
-#endif
-
 #include <set>
 
 #include <wx/frame.h>
 #include <wx/process.h>
+#include <wx/msgdlg.h>
+#include <wx/windowptr.h>
 
 class WXDLLIMPEXP_FWD_CORE wxSplitterWindow;
 class WXDLLIMPEXP_FWD_CORE wxTextCtrl;
@@ -45,12 +43,14 @@ class WXDLLIMPEXP_FWD_CORE wxStaticText;
 #include "gexecute.h"
 #include "edlistctrl.h"
 #include "edapp.h"
+#include "tm/transmem.h"
 
 class ListHandler;
 class TextctrlHandler;
 class TransTextctrlHandler;
+class SourceTextCtrl;
+class TranslationTextCtrl;
 
-class TranslationMemory;
 class PoeditFrame;
 class AttentionBar;
 class ErrorBar;
@@ -64,15 +64,23 @@ class PoeditFrame : public wxFrame
 {
     public:
         /** Public constructor functions. Creates and shows frame
-            (and optionally opens \a catalog). If \a catalog is not
-            empty and is already opened in another Poedit frame,
-            then this function won't create new frame but instead
-            return pointer to existing one.
+            and opens \a catalog. If \a catalog is already opened
+            in another Poedit frame, then this function won't create
+            new frame but instead return pointer to existing one.
 
-            \param catalog filename of catalog to open. If empty, starts
-                           w/o opened file.
+            \param catalog filename of catalog to open.
          */
-        static PoeditFrame *Create(const wxString& catalog = wxEmptyString);
+        static PoeditFrame *Create(const wxString& catalog);
+
+        /** Public constructor functions. Creates and shows frame
+            without catalog or other content.
+         */
+        static PoeditFrame *CreateEmpty();
+
+        /** Public constructor functions. Creates and shows frame
+            without catalog, just a welcome screen.
+         */
+        static PoeditFrame *CreateWelcome();
 
         /// Opens given file in this frame. Asks user for permission first
         /// if there's unsaved document.
@@ -87,18 +95,27 @@ class PoeditFrame : public wxFrame
         /// content, not having catalog loaded); NULL otherwise.
         static PoeditFrame *UnusedActiveWindow();
 
+        /// Returns true if at least one one window has unsaved changes
+        static bool AnyWindowIsModified();
+
         ~PoeditFrame();
 
-        /// Reads catalog, refreshes controls.
+        /// Reads catalog, refreshes controls, takes ownership of catalog.
         void ReadCatalog(const wxString& catalog);
+        /// Reads catalog, refreshes controls, takes ownership of catalog.
+        void ReadCatalog(Catalog *cat);
         /// Writes catalog.
-        bool WriteCatalog(const wxString& catalog);
+        void WriteCatalog(const wxString& catalog);
+
+        template<typename TFunctor>
+        void WriteCatalog(const wxString& catalog, TFunctor completionHandler);
+
         /// Did the user modify the catalog?
         bool IsModified() const { return m_modified; }
         /** Updates catalog and sets m_modified flag. Updates from POT
             if \a pot_file is not empty and from sources otherwise.
          */
-        void UpdateCatalog(const wxString& pot_file = wxEmptyString);
+        bool UpdateCatalog(const wxString& pot_file = wxEmptyString);
 
 
         virtual void DoGiveHelp(const wxString& text, bool show);
@@ -106,6 +123,9 @@ class PoeditFrame : public wxFrame
         void UpdateAfterPreferencesChange();
         static void UpdateAllAfterPreferencesChange();
 
+        void EditCatalogProperties();
+        void EditCatalogPropertiesAndUpdateFromSources();
+
     private:
         /** Ctor.
             \param catalog filename of catalog to open. If empty, starts
@@ -113,6 +133,27 @@ class PoeditFrame : public wxFrame
          */
         PoeditFrame();
 
+        /// Current kind of content view
+        enum class Content
+        {
+            Invalid, // no content whatsoever
+            Welcome,
+            PO,
+            Empty_PO
+        };
+        Content m_contentType;
+        /// parent of all content controls etc.
+        wxWindow *m_contentView;
+        wxSizer *m_contentWrappingSizer;
+
+        /// Ensures creation of specified content view, destroying the
+        /// current content if necessary
+        void EnsureContentView(Content type);
+        wxWindow* CreateContentViewPO();
+        wxWindow* CreateContentViewWelcome();
+        wxWindow* CreateContentViewEmptyPO();
+        void DestroyContentView();
+
         static PoeditFramesList ms_instances;
 
     private:
@@ -126,7 +167,11 @@ class PoeditFrame : public wxFrame
 
         // if there's modified catalog, ask user to save it; return true
         // if it's save to discard m_catalog and load new data
-        bool CanDiscardCurrentDoc();
+        template<typename TFunctor>
+        void DoIfCanDiscardCurrentDoc(TFunctor completionHandler);
+        bool NeedsToAskIfCanDiscardCurrentDoc() const;
+        wxWindowPtr<wxMessageDialog> CreateAskAboutSavingDialog();
+
         // implements opening of files, without asking user
         void DoOpenFile(const wxString& filename);
 
@@ -145,19 +190,15 @@ class PoeditFrame : public wxFrame
         /// Updates the editable nature of the comment window
         void UpdateCommentWindowEditable();
 
-        /// Returns popup menu for given catalog entry.
-        wxMenu *GetPopupMenu(size_t item);
+        // Called when catalog's language possibly changed
+        void UpdateTextLanguage();
 
-#ifdef USE_TRANSMEM
-        /// Initializes translation memory, if enabled
-        TranslationMemory *GetTransMem();
-#endif
+        /// Returns popup menu for given catalog entry.
+        wxMenu *GetPopupMenu(int item);
 
         // (Re)initializes spellchecker, if needed
         void InitSpellchecker();
 
-        void EditCatalogProperties();
-
         // navigation to another item in the list
         typedef bool (*NavigatePredicate)(const CatalogItem& item);
         void Navigate(int step, NavigatePredicate predicate, bool wrap);
@@ -172,8 +213,13 @@ class PoeditFrame : public wxFrame
         // Message handlers:
 public: // for PoeditApp
         void OnNew(wxCommandEvent& event);
+        void NewFromScratch();
+        void NewFromPOT();
+
         void OnOpen(wxCommandEvent& event);
+#ifndef __WXOSX__
         void OnOpenHist(wxCommandEvent& event);
+#endif
 private:
         void OnCloseCmd(wxCommandEvent& event);
         void OnSave(wxCommandEvent& event);
@@ -193,7 +239,7 @@ private:
         void OnRightClick(wxCommandEvent& event);
         void OnFuzzyFlag(wxCommandEvent& event);
         void OnQuotesFlag(wxCommandEvent& event);
-        void OnLinesFlag(wxCommandEvent& event);
+        void OnIDsFlag(wxCommandEvent& event);
         void OnCommentWinFlag(wxCommandEvent& event);
         void OnAutoCommentsWinFlag(wxCommandEvent& event);
         void OnCopyFromSource(wxCommandEvent& event);
@@ -207,11 +253,16 @@ private:
         void OnSortBySource(wxCommandEvent&);
         void OnSortByTranslation(wxCommandEvent&);
         void OnSortUntranslatedFirst(wxCommandEvent&);
-#ifdef USE_TRANSMEM
+
+#if defined(__WXMSW__) || defined(__WXGTK__)
+        void OnTextEditingCommand(wxCommandEvent& event);
+        void OnTextEditingCommandUpdate(wxUpdateUIEvent& event);
+#endif
+
         void OnAutoTranslate(wxCommandEvent& event);
         void OnAutoTranslateAll(wxCommandEvent& event);
-        bool AutoTranslateCatalog();
-#endif
+        bool AutoTranslateCatalog(int *matchesCount = nullptr);
+
         void OnPurgeDeleted(wxCommandEvent& event);
 
         void OnGoToBookmark(wxCommandEvent& event);
@@ -234,9 +285,14 @@ private:
 
         void RefreshSelectedItem();
 
-        void ReportValidationErrors(int errors, bool from_save);
+        template<typename TFunctor>
+        void ReportValidationErrors(int errors, Catalog::CompilationStatus mo_compilation_status,
+                                    bool from_save, TFunctor completionHandler);
 
+#ifndef __WXOSX__
         wxFileHistory& FileHistory() { return wxGetApp().FileHistory(); }
+#endif
+        void NoteAsRecentFile();
 
         DECLARE_EVENT_TABLE()
 
@@ -246,12 +302,9 @@ private:
         bool m_commentWindowEditable;
         Catalog *m_catalog;
         wxString m_fileName;
+        bool m_fileExistsOnDisk;
 
-#ifdef USE_TRANSMEM
-        TranslationMemory *m_transMem;
-        bool m_transMemLoaded;
-        wxArrayString m_autoTranslations;
-#endif
+        TranslationMemory::Results m_autoTranslations;
 
         wxPanel *m_bottomLeftPanel;
         wxPanel *m_bottomRightPanel;
@@ -260,12 +313,16 @@ private:
         wxStaticText *m_labelComment, *m_labelAutoComments;
         wxStaticText *m_labelContext;
         ErrorBar *m_errorBar;
-        wxTextCtrl *m_textOrig, *m_textOrigPlural, *m_textTrans, *m_textComment, *m_textAutoComments;
-        std::vector<wxTextCtrl*> m_textTransPlural;
-        wxTextCtrl *m_textTransSingularForm;
+        SourceTextCtrl *m_textOrig, *m_textOrigPlural;
+        wxTextCtrl *m_textComment, *m_textAutoComments;
+        TranslationTextCtrl *m_textTrans;
+        std::vector<TranslationTextCtrl*> m_textTransPlural;
+        TranslationTextCtrl *m_textTransSingularForm;
         wxNotebook *m_pluralNotebook;
         wxStaticText *m_labelSingular, *m_labelPlural;
+#ifndef __WXOSX__
         wxMenu *m_menuForHistory;
+#endif
 
         wxFont m_normalGuiFont, m_boldGuiFont;
 
@@ -274,7 +331,7 @@ private:
         bool m_modified;
         bool m_hasObsoleteItems;
         bool m_displayQuotes;
-        bool m_displayLines;
+        bool m_displayIDs;
         bool m_displayCommentWin;
         bool m_displayAutoCommentsWin;
         bool m_dontAutoclearFuzzyStatus;
