@@ -1,7 +1,7 @@
-/*
- *  This file is part of Poedit (http://www.poedit.net)
+﻿/*
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 1999-2013 Vaclav Slavik
+ *  Copyright (C) 1999-2014 Vaclav Slavik
  *  Copyright (C) 2005 Olivier Sannier
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
@@ -26,6 +26,7 @@
 
 #include "edlistctrl.h"
 
+#include "language.h"
 #include "digits.h"
 #include "cat_sorting.h"
 
@@ -67,15 +68,15 @@ inline bool IsAlmostWhite(const wxColour& clr)
 
 // colours used in the list:
 
-const wxColour gs_ErrorColor(_T("#ff5050"));
+const wxColour gs_ErrorColor("#ff5050");
 
 // colors for white list control background
-const wxColour gs_UntranslatedForWhite(_T("#103f67"));
-const wxColour gs_FuzzyForWhite(_T("#a9861b"));
+const wxColour gs_UntranslatedForWhite("#103f67");
+const wxColour gs_FuzzyForWhite("#a9861b");
 
 // ditto for black background
-const wxColour gs_UntranslatedForBlack(_T("#1962a0"));
-const wxColour gs_FuzzyForBlack(_T("#a9861b"));
+const wxColour gs_UntranslatedForBlack("#1962a0");
+const wxColour gs_FuzzyForBlack("#a9861b");
 
 
 const wxColour gs_TranspColor(254, 0, 253); // FIXME: get rid of this
@@ -104,7 +105,8 @@ wxBitmap AddDigit(int digit, int x, int y, const wxBitmap& bmp)
     wxMemoryDC dc;
     int width = bmp.GetWidth();
     int height = bmp.GetHeight();
-    wxBitmap tmpBmp(width, height);
+    wxBitmap tmpBmp;
+    tmpBmp.CreateScaled(bmp.GetScaledWidth(), bmp.GetScaledHeight(), -1, bmp.GetScaleFactor());
     dc.SelectObject(tmpBmp);
     dc.SetPen(*wxTRANSPARENT_PEN);
     dc.SetBrush(wxBrush(gs_TranspColor, wxSOLID));
@@ -130,7 +132,8 @@ wxBitmap AddDigit(int digit, int x, int y, const wxBitmap& bmp)
 wxBitmap MergeBitmaps(const wxBitmap& bmp1, const wxBitmap& bmp2)
 {
     wxMemoryDC dc;
-    wxBitmap tmpBmp(bmp1.GetWidth(), bmp1.GetHeight());
+    wxBitmap tmpBmp;
+    tmpBmp.CreateScaled(bmp1.GetScaledWidth(), bmp1.GetScaledHeight(), -1, bmp1.GetScaleFactor());
 
     dc.SelectObject(tmpBmp);
     dc.SetPen(*wxTRANSPARENT_PEN);
@@ -174,13 +177,16 @@ PoeditListCtrl::PoeditListCtrl(wxWindow *parent,
                const wxPoint &pos,
                const wxSize &size,
                long style,
-               bool dispLines,
+               bool dispIDs,
                const wxValidator& validator,
                const wxString &name)
      : wxListView(parent, id, pos, size, style | wxLC_VIRTUAL, validator, name)
 {
     m_catalog = NULL;
-    m_displayLines = dispLines;
+    m_displayIDs = dispIDs;
+
+    m_isRTL = false;
+    m_appIsRTL = (wxTheApp->GetLayoutDirection() == wxLayout_RightToLeft);
 
     sortOrder = SortOrder::Default();
 
@@ -190,24 +196,24 @@ PoeditListCtrl::PoeditListCtrl(wxWindow *parent,
     wxImageList *list = new wxImageList(16, 16);
 
     // IMG_NOTHING:
-    list->Add(wxArtProvider::GetBitmap(_T("poedit-status-nothing")));
+    list->Add(wxArtProvider::GetBitmap("poedit-status-nothing"));
 
     // IMG_AUTOMATIC:
-    list->Add(wxArtProvider::GetBitmap(_T("poedit-status-automatic")));
+    list->Add(wxArtProvider::GetBitmap("poedit-status-automatic"));
     // IMG_COMMENT:
-    list->Add(wxArtProvider::GetBitmap(_T("poedit-status-comment")));
+    list->Add(wxArtProvider::GetBitmap("poedit-status-comment"));
     // IMG_AUTOMATIC | IMG_COMMENT:
-    list->Add(MergeBitmaps(wxArtProvider::GetBitmap(_T("poedit-status-automatic")),
-                           wxArtProvider::GetBitmap(_T("poedit-status-comment"))));
+    list->Add(MergeBitmaps(wxArtProvider::GetBitmap("poedit-status-automatic"),
+                           wxArtProvider::GetBitmap("poedit-status-comment")));
 
     // IMG_MODIFIED
-    list->Add(wxArtProvider::GetBitmap(_T("poedit-status-modified")));
+    list->Add(wxArtProvider::GetBitmap("poedit-status-modified"));
 
     // IMG_MODIFIED variations:
     for (i = 1; i < IMG_MODIFIED; i++)
     {
         list->Add(MergeBitmaps(BitmapFromList(list, i),
-                               wxArtProvider::GetBitmap(_T("poedit-status-modified"))));
+                               wxArtProvider::GetBitmap("poedit-status-modified")));
     }
 
     // BK_XX variations:
@@ -244,7 +250,7 @@ PoeditListCtrl::PoeditListCtrl(wxWindow *parent,
     if ( shaded == *wxWHITE )
     {
         // use standard shaded color from finder/databrowser:
-        shaded.Set(_T("#f0f5fd"));
+        shaded.Set("#f0f5fd");
     }
     else
 #endif // __WXMAC__
@@ -282,6 +288,10 @@ PoeditListCtrl::PoeditListCtrl(wxWindow *parent,
     m_attrInvalid[0].SetBackgroundColour(gs_ErrorColor);
     m_attrInvalid[1].SetBackgroundColour(gs_ErrorColor);
 
+    // Use gray for IDs
+    if ( IsAlmostBlack(visual.colFg) )
+        m_attrId.SetTextColour(wxColour("#A1A1A1"));
+
     SetCustomFont(wxNullFont);
 }
 
@@ -315,7 +325,7 @@ void PoeditListCtrl::SetCustomFont(wxFont font_)
 
 void PoeditListCtrl::SetDisplayLines(bool dl)
 {
-    m_displayLines = dl;
+    m_displayIDs = dl;
     CreateColumns();
 }
 
@@ -324,21 +334,32 @@ void PoeditListCtrl::CreateColumns()
     DeleteAllColumns();
     InsertColumn(0, _("Source text"));
     InsertColumn(1, _("Translation"));
-    if (m_displayLines)
-        InsertColumn(2, _("Line"), wxLIST_FORMAT_RIGHT);
+    if (m_displayIDs)
+        InsertColumn(2, _("ID"), wxLIST_FORMAT_RIGHT);
+
+#ifdef __WXMSW__
+    if (m_appIsRTL)
+    {
+        // another wx quirk: if we truly need left alignment, we must lie under RTL locales
+        wxListItem colInfoOrig;
+        colInfoOrig.SetAlign(wxLIST_FORMAT_RIGHT);
+        SetColumn(0, colInfoOrig);
+    }
+#endif
+
     SizeColumns();
 }
 
 void PoeditListCtrl::SizeColumns()
 {
-    const int LINE_COL_SIZE = m_displayLines ? 50 : 0;
+    const int LINE_COL_SIZE = m_displayIDs ? 50 : 0;
 
     int w = GetSize().x
             - wxSystemSettings::GetMetric(wxSYS_VSCROLL_X) - 10
             - LINE_COL_SIZE;
     SetColumnWidth(0, w / 2);
     SetColumnWidth(1, w - w / 2);
-    if (m_displayLines)
+    if (m_displayIDs)
         SetColumnWidth(2, LINE_COL_SIZE);
 
     m_colWidth = (w/2) / GetCharWidth();
@@ -374,6 +395,22 @@ void PoeditListCtrl::ReadCatalog()
         return;
     }
 
+    auto lang = m_catalog->GetLanguage();
+    auto isRTL = lang.IsRTL();
+#ifdef __WXMSW__
+    // a quirk of wx API: if the current locale is RTL, the meaning of L and R is reversed
+    if (m_appIsRTL)
+        isRTL = !isRTL;
+#endif
+    m_isRTL = isRTL;
+
+    wxString langname = lang.IsValid() ? lang.DisplayName() : _("unknown language");
+    wxListItem colInfo;
+    colInfo.SetMask(wxLIST_MASK_TEXT);
+    colInfo.SetText(wxString::Format(_(L"Translation — %s"), langname));
+    colInfo.SetAlign(isRTL ? wxLIST_FORMAT_RIGHT : wxLIST_FORMAT_LEFT);
+    SetColumn(1, colInfo);
+
     // sort catalog items, create indexes mapping
     CreateSortMap();
 
@@ -425,11 +462,12 @@ void PoeditListCtrl::CreateSortMap()
 
     // m_mapListToCatalog will hold our desired sort order. Sort it in place
     // now, using the desired sort criteria.
+    CatalogItemsComparator comparator(*m_catalog, sortOrder);
     std::sort
     (
         m_mapListToCatalog.begin(),
         m_mapListToCatalog.end(),
-        CatalogItemsComparator(*m_catalog, sortOrder)
+        std::ref(comparator)
     );
 
     // Finally, construct m_mapCatalogToList to be the inverse mapping to
@@ -444,7 +482,7 @@ wxString PoeditListCtrl::OnGetItemText(long item, long column) const
     if (m_catalog == NULL)
         return wxEmptyString;
 
-    const CatalogItem& d = ListIndexToCatalogItem(item);
+    const CatalogItem& d = ListIndexToCatalogItem((int)item);
 
     switch (column)
     {
@@ -452,19 +490,27 @@ wxString PoeditListCtrl::OnGetItemText(long item, long column) const
         {
             wxString orig;
             if ( d.HasContext() )
-                orig.Printf(_T("%s  [ %s ]"),
+                orig.Printf("%s  [ %s ]",
                             d.GetString().c_str(), d.GetContext().c_str());
             else
                 orig = d.GetString();
-            return orig.substr(0,GetMaxColChars());
+
+            // Add RTL Unicode mark to render bidi texts correctly
+            if (m_appIsRTL)
+                return L"\u202a" + orig.substr(0,GetMaxColChars());
+            else
+                return orig.substr(0,GetMaxColChars());
         }
         case 1:
         {
-            wxString trans = d.GetTranslation();
-            return trans;
+            // Add RTL Unicode mark to render bidi texts correctly
+            if (m_isRTL && !m_appIsRTL)
+                return L"\u202b" + d.GetTranslation();
+            else
+                return d.GetTranslation();
         }
         case 2:
-            return wxString() << d.GetLineNumber();
+            return wxString() << d.GetId();
 
         default:
             return wxEmptyString;
@@ -478,7 +524,7 @@ wxListItemAttr *PoeditListCtrl::OnGetItemAttr(long item) const
     if (m_catalog == NULL)
         return (wxListItemAttr*)&m_attrNormal[idx];
 
-    const CatalogItem& d = ListIndexToCatalogItem(item);
+    const CatalogItem& d = ListIndexToCatalogItem((int)item);
 
     if (d.GetValidity() == CatalogItem::Val_Invalid)
         return (wxListItemAttr*)&m_attrInvalid[idx];
@@ -490,12 +536,24 @@ wxListItemAttr *PoeditListCtrl::OnGetItemAttr(long item) const
         return (wxListItemAttr*)&m_attrNormal[idx];
 }
 
+wxListItemAttr *PoeditListCtrl::OnGetItemColumnAttr(long item, long column) const
+{
+    if (column == 2)
+    {
+        return (wxListItemAttr*)&m_attrId;
+    }
+    else
+    {
+        return OnGetItemAttr(item);
+    }
+}
+
 int PoeditListCtrl::OnGetItemImage(long item) const
 {
     if (m_catalog == NULL)
         return IMG_NOTHING;
 
-    const CatalogItem& d = ListIndexToCatalogItem(item);
+    const CatalogItem& d = ListIndexToCatalogItem((int)item);
     int index = IMG_NOTHING;
 
     if (d.IsAutomatic())
