@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 1999-2013 Vaclav Slavik
+ *  Copyright (C) 1999-2014 Vaclav Slavik
  *  Copyright (C) 2005 Olivier Sannier
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
@@ -47,9 +47,9 @@ class PoeditListCtrl : public wxListView
                        const wxPoint &pos = wxDefaultPosition,
                        const wxSize &size = wxDefaultSize,
                        long style = wxLC_ICON,
-                       bool dispLines = false,
+                       bool dispIDs = false,
                        const wxValidator& validator = wxDefaultValidator,
-                       const wxString &name = _T("listctrl"));
+                       const wxString &name = "listctrl");
 
         virtual ~PoeditListCtrl();
 
@@ -63,7 +63,8 @@ class PoeditListCtrl : public wxListView
         void CatalogChanged(Catalog* catalog);
 
         virtual wxString OnGetItemText(long item, long column) const;
-        virtual wxListItemAttr * OnGetItemAttr(long item) const;
+        virtual wxListItemAttr *OnGetItemAttr(long item) const;
+        virtual wxListItemAttr *OnGetItemColumnAttr(long item, long column) const;
         virtual int OnGetItemImage(long item) const;
 
         /// Returns the list item index for the given catalog index
@@ -92,7 +93,7 @@ class PoeditListCtrl : public wxListView
 
 
         /// Returns current list selection (as list item index)
-        int GetSelection() const { return GetFirstSelected(); }
+        int GetSelection() const { return (int)GetFirstSelected(); }
 
         /// Returns index of selected catalog item
         int GetSelectedCatalogItem() const
@@ -132,14 +133,16 @@ class PoeditListCtrl : public wxListView
         void ReadCatalog();
         void OnSize(wxSizeEvent& event);
 
-        bool m_displayLines;
+        bool m_displayIDs;
         unsigned m_colWidth;
+        bool m_isRTL, m_appIsRTL;
 
         Catalog* m_catalog;
 
         std::vector<int> m_mapListToCatalog;
         std::vector<int> m_mapCatalogToList;
 
+        wxListItemAttr m_attrId;
         wxListItemAttr m_attrNormal[2];
         wxListItemAttr m_attrUntranslated[2];
         wxListItemAttr m_attrFuzzy[2];
