@@ -1,7 +1,8 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
  *  Copyright (C) 2003 Christophe Hermier
+ *  Copyright (C) 2013-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -27,6 +28,7 @@
 #include <wx/colour.h>
 #include <wx/utils.h>
 #include <wx/textfile.h>
+#include <wx/log.h>
 
 #include "catalog.h"
 #include "utility.h"
@@ -49,6 +51,12 @@ wxColour
     g_ItemColourUntranslated[2] = LIST_COLOURS(0xA5,0xEA,0xEF), // blue
     g_ItemColourFuzzy[2] =        LIST_COLOURS(0xF4,0xF1,0xC1); // yellow
 
+wxString FormatTransString(const wxString& s)
+{
+    wxString out = EscapeMarkup(s);
+    out.Replace("\\n", "\\n<br>");
+    return out;
+}
 
 } // anonymous namespace
 
@@ -57,12 +65,10 @@ bool Catalog::ExportToHTML(const wxString& filename)
     size_t i;
     wxTextFile f;
 
-    if ( wxFileExists(filename) )
-    {
-        wxRemoveFile ( filename);
-    }
+    TempOutputFileFor tempfile_obj(filename);
+    const wxString tempfile = tempfile_obj.FileName();
 
-    if (!f.Create(filename))
+    if (!f.Create(tempfile))
     {
         return false;
     }
@@ -73,20 +79,20 @@ bool Catalog::ExportToHTML(const wxString& filename)
 
     // HTML HEADER
     f.AddLine(_T("<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.01 Transitional//EN\" \"http://www.w3.org/TR/html4/loose.dtd\">"));
-    f.AddLine(_T("<html>"));
+    f.AddLine("<html>");
 
-    f.AddLine(_T("<head>"));
-    line.Printf(_T("<title> %s - %s - Poedit Export </title>"),
+    f.AddLine("<head>");
+    line.Printf("<title> %s - %s - Poedit Export </title>",
                 EscapeMarkup(m_header.Project).c_str(),
-                EscapeMarkup(m_header.LanguageCode).c_str());
+                EscapeMarkup(m_header.Lang.Code()).c_str());
     f.AddLine(line);
     f.AddLine(_T("<meta http-equiv=\"Content-Type\" content=\"text/html; charset=utf-8\">" ) );
-    f.AddLine(_T("</head>"));
-    f.AddLine(_T("<body bgcolor='#FFFFFF'>"));
+    f.AddLine("</head>");
+    f.AddLine("<body bgcolor='#FFFFFF'>");
 
-    line.Printf(_T("<h1> %s : %s</h1>"),
+    line.Printf("<h1> %s : %s</h1>",
                 EscapeMarkup(m_header.Project).c_str(),
-                EscapeMarkup(m_header.LanguageCode).c_str());
+                EscapeMarkup(m_header.Lang.Code()).c_str());
     f.AddLine(line);
 
 
@@ -95,18 +101,18 @@ bool Catalog::ExportToHTML(const wxString& filename)
     // String here are duplicates from the ones in setting.xrc
     // TODO find a way if possible to synchronize them
 
-    f.AddLine(_T("<table align=center border=1 cellspacing=2 cellpadding=4>"));
+    f.AddLine("<table align=center border=1 cellspacing=2 cellpadding=4>");
 
-    line.Printf(_T("<tr><th colspan=2>%s</th></tr>"),
+    line.Printf("<tr><th colspan=2>%s</th></tr>",
                 _("Project info"));
     f.AddLine(line);
-    wxString line_format = _T("<tr><td>%s</td><td>%s</td></tr>");
+    wxString line_format = "<tr><td>%s</td><td>%s</td></tr>";
     line.Printf(line_format,
                 _("Project name and version:"),
                 EscapeMarkup(m_header.Project).c_str());
     f.AddLine(line);
     line.Printf(line_format, _("Language:"),
-                EscapeMarkup(m_header.LanguageCode).c_str());
+                EscapeMarkup(m_header.Lang.Code()).c_str());
     f.AddLine(line);
     line.Printf(line_format, _("Team:"),
                 EscapeMarkup(m_header.Team).c_str());
@@ -120,7 +126,7 @@ bool Catalog::ExportToHTML(const wxString& filename)
                 EscapeMarkup(m_header.Charset).c_str());
     f.AddLine(line);
 
-    f.AddLine( _T("</table>") );
+    f.AddLine( "</table>" );
     // statistics
 
     int all = 0;
@@ -131,85 +137,146 @@ bool Catalog::ExportToHTML(const wxString& filename)
     GetStatistics(&all, &fuzzy, &badtokens, &untranslated, &unfinished);
 
     int percent = (all == 0 ) ? 0 : (100 * (all - unfinished) / all);
-    line.Printf(_("%i %% translated, %i strings (%i fuzzy, %i bad tokens, %i not translated)"),
-               percent, all, fuzzy, badtokens, untranslated);
+
+    wxString details;
+    if ( fuzzy > 0 )
+    {
+        details += wxString::Format(wxPLURAL("%i fuzzy", "%i fuzzy", fuzzy), fuzzy);
+    }
+    if ( badtokens > 0 )
+    {
+        if ( !details.empty() )
+            details += ", ";
+        details += wxString::Format(wxPLURAL("%i bad token", "%i bad tokens", badtokens), badtokens);
+    }
+    if ( untranslated > 0 )
+    {
+        if ( !details.empty() )
+            details += ", ";
+        details += wxString::Format(wxPLURAL("%i not translated", "%i not translated", untranslated), untranslated);
+    }
+    if ( details.empty() )
+    {
+        line.Printf(wxPLURAL("%i %% translated, %i string", "%i %% translated, %i strings", all),
+                    percent, all);
+    }
+    else
+    {
+        line.Printf(wxPLURAL("%i %% translated, %i string (%s)", "%i %% translated, %i strings (%s)", all),
+                    percent, all, details.c_str());
+    }
 
     f.AddLine(line);
 
 
     // data printed in a table :
-    f.AddLine(_T("<table border=1 cellspacing=2 cellpadding=4>"));
+    f.AddLine("<table border=1 cellspacing=2 cellpadding=4>");
 
-    f.AddLine(_T("<tr>"));
-    f.AddLine(_T("<th>"));
+    f.AddLine("<tr>");
+    f.AddLine("<th>");
     f.AddLine(_("Source"));
-    f.AddLine(_T("</th>"));
-    f.AddLine(_T("<th>"));
+    f.AddLine("</th>");
+    f.AddLine("<th>");
     f.AddLine(_("Translation"));
-    f.AddLine(_T("</th>"));
-    f.AddLine(_T("<th>"));
+    f.AddLine("</th>");
+    f.AddLine("<th>");
     f.AddLine(_("Notes"));
-    f.AddLine(_T("</th>"));
-    f.AddLine(_T("</tr>"));
+    f.AddLine("</th>");
+    f.AddLine("</tr>");
 
     for (i = 0; i < GetCount(); i++)
     {
         const CatalogItem& data = m_items[i];
 
         wxColour bgcolor = g_ItemColourNormal[i % 2];
-        wxString source_string = data.GetString();
 
-        wxString translation = data.GetTranslation();
+        wxString source_string = FormatTransString(data.GetString());
+        if (data.HasContext())
+            source_string += FormatTransString(wxString::Format("  [ %s ]", data.GetContext()));
+        if (data.HasPlural())
+            source_string += "<br>~~~<br>\n" + FormatTransString(data.GetPluralString());
+
+        wxString translation = FormatTransString(data.GetTranslation());
+        if (data.HasPlural())
+        {
+            for (unsigned int t = 1; t < data.GetNumberOfTranslations(); t++)
+                translation += "<br>~~~<br>\n" + FormatTransString(data.GetTranslation(t));
+        }
+
         if (translation.empty())
         {
-            translation = _T(" ");
+            translation = " ";
             bgcolor = g_ItemColourUntranslated[i % 2];
         }
 
-        wxString flags;
+        wxString notes;
 
         if (data.IsAutomatic())
         {
-            flags += EscapeMarkup(_("Automatic translation"));
-            flags += _T("<BR>");
+            notes += EscapeMarkup(_("Automatic translation"));
+            notes += "<BR>";
         }
         if (data.IsFuzzy())
         {
             bgcolor = g_ItemColourFuzzy[i % 2];
-            flags += EscapeMarkup(_("Fuzzy translation"));
-            flags += _T("<BR>");
-        }
-        if (flags.empty())
-        {
-            flags = _T(" ");
+            notes += EscapeMarkup(_("Fuzzy translation"));
+            notes += "<BR>";
         }
 
+        if (data.HasAutoComments())
+        {
+            notes += "<font color='#888'>";
+            notes += _("Notes for translators:");
+            notes += "</font>";
+            notes += "<br>";
+            for (auto n: data.GetAutoComments())
+                notes += EscapeMarkup(n) + "<br>";
+        }
+        if (data.HasComment())
+        {
+            notes += "<font color='#888'>";
+            notes += _("Comment:");
+            notes += "</font>";
+            notes += "<br>";
+            notes += EscapeMarkup(data.GetComment());
+            notes += "<br>";
+        }
+
+        if (notes.empty())
+            notes = " ";
+
         wxString tr;
-        tr.Printf(_T("<tr bgcolor='#%0X%0X%0X'>"),
+        tr.Printf("<tr bgcolor='#%0X%0X%0X'>",
                   bgcolor.Red(), bgcolor.Green(), bgcolor.Blue());
         f.AddLine(tr);
 
-        f.AddLine(_T("<td>"));
-        f.AddLine(EscapeMarkup(source_string));
-        f.AddLine(_T("</td>"));
-        f.AddLine(_T("<td>"));
-        f.AddLine(EscapeMarkup(translation));
-        f.AddLine(_T("</td>"));
-        f.AddLine(_T("<td>"));
+        f.AddLine("<td>");
+        f.AddLine(source_string);
+        f.AddLine("</td>");
+        f.AddLine("<td>");
+        f.AddLine(translation);
+        f.AddLine("</td>");
+        f.AddLine("<td>");
         f.AddLine(_T("<font size=\"-1\">"));
-        f.AddLine(flags);
-        f.AddLine(_T("</font>"));
-        f.AddLine(_T("</td>"));
-        f.AddLine(_T("</tr>"));
+        f.AddLine(notes);
+        f.AddLine("</font>");
+        f.AddLine("</td>");
+        f.AddLine("</tr>");
     }
 
-    f.AddLine(_T("</table>"));
-    f.AddLine(_T("</body>"));
-    f.AddLine(_T("</html>"));
+    f.AddLine("</table>");
+    f.AddLine("</body>");
+    f.AddLine("</html>");
 
     bool written = f.Write(wxTextFileType_None, wxConvUTF8);
 
     f.Close();
 
+    if ( written && !wxRenameFile(tempfile, filename, /*overwrite=*/true) )
+    {
+        wxLogError(_("Couldn't save file %s."), filename.c_str());
+        written = false;
+    }
+
     return written;
 }
