@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 1999-2013 Vaclav Slavik
+ *  Copyright (C) 1999-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -25,6 +25,7 @@
 
 #include <wx/filename.h>
 #include <wx/log.h>
+#include <wx/button.h>
 #include <wx/panel.h>
 #include <wx/stattext.h>
 #include <wx/choice.h>
@@ -34,6 +35,7 @@
 #include <wx/listctrl.h>
 #include <wx/fontenum.h>
 #include <wx/ffile.h>
+#include <wx/utils.h>
 #include <wx/stc/stc.h>
 
 #include "fileviewer.h"
@@ -42,13 +44,13 @@
 FileViewer::FileViewer(wxWindow *parent,
                        const wxString& basePath,
                        const wxArrayString& references,
-                       size_t startAt)
+                       int startAt)
         : wxFrame(parent, -1, _("Source file")),
           m_references(references)
 {
     m_basePath = basePath;
 
-    SetName(_T("fileviewer"));
+    SetName("fileviewer");
 
     wxPanel *panel = new wxPanel(this, -1);
     wxSizer *sizer = new wxBoxSizer(wxVERTICAL);
@@ -68,6 +70,9 @@ FileViewer::FileViewer(wxWindow *parent,
         choice->Append(references[i]);
     choice->SetSelection(startAt);
 
+    wxButton *edit = new wxButton(panel, wxID_ANY, _("Open In Editor"));
+    barsizer->Add(edit, wxSizerFlags().Center().Border(wxLEFT, 10));
+
     m_text = new wxStyledTextCtrl(panel, wxID_ANY,
                                   wxDefaultPosition, wxDefaultSize,
                                   wxBORDER_THEME);
@@ -81,7 +86,20 @@ FileViewer::FileViewer(wxWindow *parent,
     SetSizer(topsizer);
     Layout();
 
+    choice->Bind(wxEVT_CHOICE, &FileViewer::OnChoice, this);
+    edit->Bind(wxEVT_BUTTON, &FileViewer::OnEditFile, this);
+
     ShowReference(m_references[startAt]);
+
+#ifdef __WXOSX__
+    wxAcceleratorEntry entries[] = {
+        { wxACCEL_CMD,  'W', wxID_CLOSE }
+    };
+    wxAcceleratorTable accel(WXSIZEOF(entries), entries);
+    SetAcceleratorTable(accel);
+
+    Bind(wxEVT_MENU, [=](wxCommandEvent&){ Destroy(); }, wxID_CLOSE);
+#endif
 }
 
 
@@ -98,21 +116,21 @@ void FileViewer::SetupTextCtrl()
     wxFont font = wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
 
 #ifdef __WXGTK__
-    font.SetFaceName(_T("monospace"));
+    font.SetFaceName("monospace");
 #else
-    static const wxChar *s_monospaced_fonts[] = {
+    static const char *s_monospaced_fonts[] = {
 #ifdef __WXMSW__
-        _T("Consolas"),
-        _T("Lucida Console"),
+        "Consolas",
+        "Lucida Console",
 #endif
 #ifdef __WXOSX__
-        _T("Menlo"),
-        _T("Monaco"),
+        "Menlo",
+        "Monaco",
 #endif
         NULL
     };
 
-    for ( const wxChar **f = s_monospaced_fonts; *f; ++f )
+    for ( const char **f = s_monospaced_fonts; *f; ++f )
     {
         if ( wxFontEnumerator::IsValidFacename(*f) )
         {
@@ -123,14 +141,14 @@ void FileViewer::SetupTextCtrl()
 #endif
 
     // style used:
-    wxString fontspec = wxString::Format(_T("face:%s,size:%d"),
+    wxString fontspec = wxString::Format("face:%s,size:%d",
                                          font.GetFaceName().c_str(),
                                          font.GetPointSize());
-    const wxString DEFAULT     = fontspec + _T(",fore:black,back:white");
-    const wxString STRING      = fontspec + _T(",bold,fore:#882d21");
-    const wxString COMMENT     = fontspec + _T(",fore:#487e18");
-    const wxString KEYWORD     = fontspec + _T(",fore:#2f00f9");
-    const wxString LINENUMBERS = fontspec + _T(",fore:#5d8bab");
+    const wxString DEFAULT     = fontspec + ",fore:black,back:white";
+    const wxString STRING      = fontspec + ",bold,fore:#882d21";
+    const wxString COMMENT     = fontspec + ",fore:#487e18";
+    const wxString KEYWORD     = fontspec + ",fore:#2f00f9";
+    const wxString LINENUMBERS = fontspec + ",fore:#5d8bab";
 
 
     // current line marker
@@ -143,7 +161,7 @@ void FileViewer::SetupTextCtrl()
     // line numbers margin size:
     t.SetMarginType(0, wxSTC_MARGIN_NUMBER);
     t.SetMarginWidth(0,
-                     t.TextWidth(wxSTC_STYLE_LINENUMBER, _T("9999 ")));
+                     t.TextWidth(wxSTC_STYLE_LINENUMBER, "9999 "));
     t.SetMarginWidth(1, 0);
     t.SetMarginWidth(2, 3);
 
@@ -170,12 +188,10 @@ void FileViewer::SetupTextCtrl()
     t.StyleSetSpec(wxSTC_TCL_COMMENTLINE, COMMENT);
     t.StyleSetSpec(wxSTC_TCL_BLOCK_COMMENT, COMMENT);
 
-#if wxCHECK_VERSION(2,9,0)
     t.StyleSetSpec(wxSTC_PAS_STRING, STRING);
     t.StyleSetSpec(wxSTC_PAS_COMMENT, COMMENT);
     t.StyleSetSpec(wxSTC_PAS_COMMENT2, COMMENT);
     t.StyleSetSpec(wxSTC_PAS_COMMENTLINE, COMMENT);
-#endif
 }
 
 
@@ -216,7 +232,7 @@ int FileViewer::GetLexer(const wxString& ext)
 }
 
 
-void FileViewer::ShowReference(wxString ref)
+wxFileName FileViewer::GetFilename(wxString ref) const
 {
     if ( ref.length() >= 3 &&
          ref[1] == _T(':') &&
@@ -224,7 +240,7 @@ void FileViewer::ShowReference(wxString ref)
     {
         // This is an absolute Windows path (c:\foo... or c:/foo...); fix
         // the latter case.
-        ref.Replace(_T("/"), _T("\\"));
+        ref.Replace("/", "\\");
     }
 
     wxPathFormat pathfmt = ref.Contains(_T('\\')) ? wxPATH_WIN : wxPATH_UNIX;
@@ -255,7 +271,7 @@ void FileViewer::ShowReference(wxString ref)
             else
             {
                 // remove the last path component
-                size_t last = basePath.find_last_of(_T("\\/"));
+                size_t last = basePath.find_last_of("\\/");
                 if ( last == wxString::npos )
                     break;
                 else
@@ -264,6 +280,13 @@ void FileViewer::ShowReference(wxString ref)
         }
     }
 
+    return filename;
+}
+
+
+void FileViewer::ShowReference(const wxString& ref)
+{
+    const wxFileName filename = GetFilename(ref);
     wxFFile file;
     wxString data;
 
@@ -290,22 +313,22 @@ void FileViewer::ShowReference(wxString ref)
     m_text->SetReadOnly(true);
 
     m_text->MarkerDeleteAll(1);
-    m_text->MarkerAdd(linenum - 1, 1);
+    m_text->MarkerAdd((int)linenum - 1, 1);
 
     // Center the highlighted line:
-    int lineHeight = m_text->TextHeight(linenum);
+    int lineHeight = m_text->TextHeight((int)linenum);
     int linesInWnd = m_text->GetSize().y / lineHeight;
-    m_text->ScrollToLine(wxMax(0, linenum - linesInWnd/2));
+    m_text->ScrollToLine(wxMax(0, (int)linenum - linesInWnd/2));
 
 }
 
 
-
-BEGIN_EVENT_TABLE(FileViewer, wxFrame)
-    EVT_CHOICE(wxID_ANY, FileViewer::OnChoice)
-END_EVENT_TABLE()
-
 void FileViewer::OnChoice(wxCommandEvent &event)
 {
     ShowReference(m_references[event.GetSelection()]);
 }
+
+void FileViewer::OnEditFile(wxCommandEvent&)
+{
+    wxLaunchDefaultApplication(GetFilename(m_current).GetFullPath());
+}
\ No newline at end of file
