@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 1999-2013 Vaclav Slavik
+ *  Copyright (C) 1999-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -30,6 +30,7 @@
 
 class WXDLLIMPEXP_FWD_CORE wxListCtrl;
 class WXDLLIMPEXP_FWD_CORE wxStyledTextCtrl;
+class WXDLLIMPEXP_FWD_CORE wxFileName;
 
 
 /** This class implements frame that shows part of file
@@ -47,17 +48,18 @@ public:
                           by default
      */
     FileViewer(wxWindow *parent, const wxString& basePath,
-               const wxArrayString& references, size_t openAt);
+               const wxArrayString& references, int openAt);
     ~FileViewer();
 
     /// Shows given reference, i.e. loads the file
-    void ShowReference(wxString ref);
+    void ShowReference(const wxString& ref);
 
     bool FileOk() { return !m_current.empty(); }
 
 private:
     void SetupTextCtrl();
     int GetLexer(const wxString& extension);
+    wxFileName GetFilename(wxString ref) const;
 
 private:
     wxString m_basePath;
@@ -68,7 +70,6 @@ private:
 
     void OnChoice(wxCommandEvent &event);
     void OnEditFile(wxCommandEvent &event);
-    DECLARE_EVENT_TABLE();
 };
 
 #endif // _FILEVIEWER_H_
