@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2001-2013 Vaclav Slavik
+ *  Copyright (C) 2001-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -24,6 +24,7 @@
  */
 
 #include <wx/xrc/xmlres.h>
+#include <wx/accel.h>
 #include <wx/config.h>
 #include <wx/button.h>
 #include <wx/textctrl.h>
@@ -39,6 +40,7 @@
 static const wxString SEPARATORS = wxT(" \t\r\n\\/:;.,?!\"'_|-+=(){}[]<>&#@");
 
 wxString FindFrame::ms_text;
+FindFrame *FindFrame::ms_singleton = nullptr;
 
 BEGIN_EVENT_TABLE(FindFrame, wxDialog)
    EVT_BUTTON(XRCID("find_next"), FindFrame::OnNext)
@@ -63,7 +65,7 @@ FindFrame::FindFrame(wxWindow *parent,
           m_textCtrlComments(textCtrlComments),
           m_textCtrlAutoComments(textCtrlAutoComments)
 {
-    wxXmlResource::Get()->LoadDialog(this, parent, _T("find_frame"));
+    wxXmlResource::Get()->LoadDialog(this, parent, "find_frame");
 
     m_textField = XRCCTRL(*this, "string_to_find", wxTextCtrl);
 
@@ -83,62 +85,110 @@ FindFrame::FindFrame(wxWindow *parent,
     Reset(c);
 
     XRCCTRL(*this, "in_orig", wxCheckBox)->SetValue(
-        wxConfig::Get()->Read(_T("find_in_orig"), (long)true));
+        wxConfig::Get()->ReadBool("find_in_orig", true));
     XRCCTRL(*this, "in_trans", wxCheckBox)->SetValue(
-        wxConfig::Get()->Read(_T("find_in_trans"), (long)true));
+        wxConfig::Get()->ReadBool("find_in_trans", true));
     XRCCTRL(*this, "in_comments", wxCheckBox)->SetValue(
-        wxConfig::Get()->Read(_T("find_in_comments"), (long)true));
+        wxConfig::Get()->ReadBool("find_in_comments", true));
     XRCCTRL(*this, "in_auto_comments", wxCheckBox)->SetValue(
-        wxConfig::Get()->Read(_T("find_in_auto_comments"), (long)true));
+        wxConfig::Get()->ReadBool("find_in_auto_comments", true));
     XRCCTRL(*this, "case_sensitive", wxCheckBox)->SetValue(
-        wxConfig::Get()->Read(_T("find_case_sensitive"), (long)false));
+        wxConfig::Get()->ReadBool("find_case_sensitive", false));
     XRCCTRL(*this, "from_first", wxCheckBox)->SetValue(
-        wxConfig::Get()->Read(_T("find_from_first"), (long)true));
+        wxConfig::Get()->ReadBool("find_from_first", true));
     XRCCTRL(*this, "whole_words", wxCheckBox)->SetValue(
-        wxConfig::Get()->Read(_T("whole_words"), (long)false));
+        wxConfig::Get()->ReadBool("whole_words", false));
+
+#ifdef __WXOSX__
+    wxAcceleratorEntry entries[] = {
+        { wxACCEL_CMD,  'W', wxID_CLOSE }
+    };
+    wxAcceleratorTable accel(WXSIZEOF(entries), entries);
+    SetAcceleratorTable(accel);
+#endif
+
+    ms_singleton = this;
 }
 
 
 FindFrame::~FindFrame()
 {
+    ms_singleton = nullptr;
+
     SaveWindowState(this, WinState_Pos);
 
-    wxConfig::Get()->Write(_T("find_in_orig"),
+    wxConfig::Get()->Write("find_in_orig",
             XRCCTRL(*this, "in_orig", wxCheckBox)->GetValue());
-    wxConfig::Get()->Write(_T("find_in_trans"),
+    wxConfig::Get()->Write("find_in_trans",
                 XRCCTRL(*this, "in_trans", wxCheckBox)->GetValue());
-    wxConfig::Get()->Write(_T("find_in_comments"),
+    wxConfig::Get()->Write("find_in_comments",
                 XRCCTRL(*this, "in_comments", wxCheckBox)->GetValue());
-    wxConfig::Get()->Write(_T("find_in_auto_comments"),
+    wxConfig::Get()->Write("find_in_auto_comments",
                 XRCCTRL(*this, "in_auto_comments", wxCheckBox)->GetValue());
-    wxConfig::Get()->Write(_T("find_case_sensitive"),
+    wxConfig::Get()->Write("find_case_sensitive",
                 XRCCTRL(*this, "case_sensitive", wxCheckBox)->GetValue());
-    wxConfig::Get()->Write(_T("find_from_first"),
+    wxConfig::Get()->Write("find_from_first",
                 XRCCTRL(*this, "from_first", wxCheckBox)->GetValue());
-    wxConfig::Get()->Write(_T("whole_words"),
+    wxConfig::Get()->Write("whole_words",
                 XRCCTRL(*this, "whole_words", wxCheckBox)->GetValue());
 }
 
+FindFrame *FindFrame::Get(PoeditListCtrl *list, Catalog *forCatalog)
+{
+    if (!ms_singleton)
+        return nullptr;
+    if (ms_singleton->m_catalog != forCatalog)
+    {
+        ms_singleton->m_listCtrl = list;
+        ms_singleton->Reset(forCatalog);
+    }
+    return ms_singleton;
+}
+
+
+void FindFrame::NotifyParentDestroyed(PoeditListCtrl *list, Catalog *forCatalog)
+{
+    if (!ms_singleton)
+        return;
+    if (ms_singleton->m_catalog == forCatalog || ms_singleton->m_listCtrl == list)
+    {
+        ms_singleton->Destroy();
+        ms_singleton = nullptr;
+    }
+}
+
+
 
 void FindFrame::Reset(Catalog *c)
 {
+    if (!m_listCtrl)
+        return;
+
     bool fromFirst = XRCCTRL(*this, "from_first", wxCheckBox)->GetValue();
 
     m_catalog = c;
     m_position = -1;
     if (!fromFirst)
-        m_position = m_listCtrl->GetNextItem(-1,
-                                             wxLIST_NEXT_ALL,
-                                             wxLIST_STATE_SELECTED);
+        m_position = (int)m_listCtrl->GetNextItem(-1,
+                                                  wxLIST_NEXT_ALL,
+                                                  wxLIST_STATE_SELECTED);
 
     m_btnPrev->Enable(!ms_text.empty());
     m_btnNext->Enable(!ms_text.empty());
 }
 
 
+void FindFrame::FocusSearchField()
+{
+    m_textField->SetFocus();
+    m_textField->SelectAll();
+}
+
+
 void FindFrame::OnClose(wxCommandEvent&)
 {
     Destroy();
+    ms_singleton = nullptr;
 }
 
 
@@ -200,7 +250,7 @@ bool TextInString(const wxString& str, const wxString& text, bool wholeWords)
     {
         if (wholeWords)
         {
-            unsigned textLen = text.Length();
+            size_t textLen = text.Length();
 
             bool result = true;
             if (index >0)
@@ -223,6 +273,9 @@ bool TextInString(const wxString& str, const wxString& text, bool wholeWords)
 
 bool FindFrame::DoFind(int dir)
 {
+    if (!m_listCtrl)
+        return false;
+
     int cnt = m_listCtrl->GetItemCount();
     bool inStr = XRCCTRL(*this, "in_orig", wxCheckBox)->GetValue();
     bool inTrans = XRCCTRL(*this, "in_trans", wxCheckBox)->GetValue();
@@ -257,9 +310,9 @@ bool FindFrame::DoFind(int dir)
             if (!caseSens)
                 textc.MakeLower();
             if (ignoreMnemonicsAmp)
-                textc.Replace(_T("&"), _T(""));
+                textc.Replace("&", "");
             if (ignoreMnemonicsUnderscore)
-                textc.Replace(_T("_"), _T(""));
+                textc.Replace("_", "");
             if (TextInString(textc, text, wholeWords))
             {
                 found = Found_InOrig;
@@ -269,9 +322,9 @@ bool FindFrame::DoFind(int dir)
         if (inTrans)
         {
             // concatenate all translations:
-            size_t cnt = dt.GetNumberOfTranslations();
+            unsigned cntTrans = dt.GetNumberOfTranslations();
             textc = wxEmptyString;
-            for (size_t i = 0; i < cnt; i++)
+            for (unsigned i = 0; i < cntTrans; i++)
             {
                 textc += dt.GetTranslation(i);
             }
@@ -279,9 +332,9 @@ bool FindFrame::DoFind(int dir)
             if (!caseSens)
                 textc.MakeLower();
             if (ignoreMnemonicsAmp)
-                textc.Replace(_T("&"), _T(""));
+                textc.Replace("&", "");
             if (ignoreMnemonicsUnderscore)
-                textc.Replace(_T("_"), _T(""));
+                textc.Replace("_", "");
 
             if (TextInString(textc, text, wholeWords)) { found = Found_InTrans; break; }
         }
@@ -321,7 +374,7 @@ bool FindFrame::DoFind(int dir)
 
         // find the text on the control and select it:
 
-        wxTextCtrl* txt;
+        wxTextCtrl* txt = NULL;
         switch (found)
         {
             case Found_InOrig:
