@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2001-2013 Vaclav Slavik
+ *  Copyright (C) 2001-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -26,11 +26,13 @@
 #ifndef _FINDFRAME_H_
 #define _FINDFRAME_H_
 
+#include "edlistctrl.h"
+
 #include <wx/dialog.h>
+#include <wx/weakref.h>
 
 class WXDLLIMPEXP_FWD_CORE wxButton;
 
-class PoeditListCtrl;
 class Catalog;
 
 /** FindFrame is small dialog frame that contains controls for searching
@@ -53,13 +55,19 @@ class FindFrame : public wxDialog
                   wxTextCtrl *textCtrlComments, wxTextCtrl *textCtrlAutoComments);
         ~FindFrame();
 
+        /// Returns singleton instance if it exists. Updates the catalog
+        /// if it differs from the current one.
+        static FindFrame *Get(PoeditListCtrl *list, Catalog *forCatalog);
+
+        static void NotifyParentDestroyed(PoeditListCtrl *list, Catalog *forCatalog);
+
         /** Resets the search to starting position and changes
             the catalog in use. Called by EditorFrame when the user
             reloads catalog.
          */
         void Reset(Catalog *c);
 
-        void FocusSearchField() { m_textField->SetFocus(); }
+        void FocusSearchField();
 
         void FindPrev();
         void FindNext();
@@ -75,7 +83,7 @@ class FindFrame : public wxDialog
 
         wxTextCtrl *m_textField;
 
-        PoeditListCtrl *m_listCtrl;
+        wxWeakRef<PoeditListCtrl> m_listCtrl;
         Catalog *m_catalog;
         int m_position;
         wxButton *m_btnPrev, *m_btnNext;
@@ -84,6 +92,8 @@ class FindFrame : public wxDialog
 
         // NB: this is static so that last search term is remembered
         static wxString ms_text;
+
+        static FindFrame *ms_singleton;
 };
 
 
