@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2000-2013 Vaclav Slavik
+ *  Copyright (C) 2000-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -34,14 +34,36 @@
 #include <wx/filename.h>
 
 #include "gexecute.h"
+#include "errors.h"
+#include "chooselang.h"
+
+namespace
+{
 
 #if defined(__WXMAC__) || defined(__WXMSW__)
-static wxString GetPathToAuxBinary(const wxString& program)
+
+wxString GetGettextPackagePath()
 {
-    wxFileName path(wxStandardPaths::Get().GetExecutablePath());
+#ifdef __WXMAC__
+    wxString dir = wxStandardPaths::Get().GetPluginsDir();
+    return dir + "/GettextTools.bundle/Contents/MacOS";
+#else
+    return wxStandardPaths::Get().GetDataDir() + wxFILE_SEP_PATH + "GettextTools";
+#endif
+}
+
+inline wxString GetAuxBinariesDir()
+{
+    return GetGettextPackagePath() + "/bin";
+}
+
+wxString GetPathToAuxBinary(const wxString& program)
+{
+    wxFileName path;
+    path.SetPath(GetAuxBinariesDir());
     path.SetName(program);
 #ifdef __WXMSW__
-    path.SetExt(_T("exe"));
+    path.SetExt("exe");
 #endif
     if ( path.IsFileExecutable() )
     {
@@ -49,8 +71,8 @@ static wxString GetPathToAuxBinary(const wxString& program)
     }
     else
     {
-        wxLogTrace(_T("poedit.execute"),
-                   _T("%s doesn't exist, falling back to %s"),
+        wxLogTrace("poedit.execute",
+                   "%s doesn't exist, falling back to %s",
                    path.GetFullPath().c_str(),
                    program.c_str());
         return program;
@@ -59,45 +81,74 @@ static wxString GetPathToAuxBinary(const wxString& program)
 #endif // __WXMAC__ || __WXMSW__
 
 
-int DoExecuteGettext(const wxString& cmdline_,
-                     wxArrayString& gstdout,
-                     wxArrayString& gstderr)
+bool ReadOutput(wxInputStream& s, wxArrayString& out)
 {
+    // the stream could be already at EOF or in wxSTREAM_BROKEN_PIPE state
+    s.Reset();
+    wxTextInputStream tis(s, " ", wxConvUTF8);
+
+    while (true)
+    {
+        wxString line = tis.ReadLine();
+        if ( !line.empty() )
+            out.push_back(line);
+        if (s.Eof())
+            break;
+        if ( !s )
+            return false;
+    }
+
+    return true;
+}
+
+long DoExecuteGettext(const wxString& cmdline_, wxArrayString& gstderr)
+{
+    wxExecuteEnv env;
     wxString cmdline(cmdline_);
 
 #if defined(__WXMAC__) || defined(__WXMSW__)
     wxString binary = cmdline.BeforeFirst(_T(' '));
     cmdline = GetPathToAuxBinary(binary) + cmdline.Mid(binary.length());
-#endif
+    wxGetEnvMap(&env.env);
+    env.env["POEDIT_USE_UTF8"] = "1";
+    env.env["POEDIT_LOCALEDIR"] = GetGettextPackagePath() + "/share/locale";
+    #if NEED_CHOOSELANG_UI
+	wxString lang = GetUILanguage();
+	if ( !lang.empty() )
+		env.env["LANG"] = lang;
+    #endif
+#endif // __WXMAC__ || __WXMSW__
 
-    wxLogTrace(_T("poedit.execute"), _T("executing: %s"), cmdline.c_str());
+    wxLogTrace("poedit.execute", "executing: %s", cmdline.c_str());
 
-#if wxCHECK_VERSION(2,9,0)
-    int retcode = wxExecute(cmdline, gstdout, gstderr, wxEXEC_BLOCK);
-#else
-    int retcode = wxExecute(cmdline, gstdout, gstderr);
-#endif
+    wxScopedPtr<wxProcess> process(new wxProcess);
+    process->Redirect();
+
+    long retcode = wxExecute(cmdline, wxEXEC_BLOCK, process.get(), &env);
+
+	wxInputStream *std_err = process->GetErrorStream();
+    if ( std_err && !ReadOutput(*std_err, gstderr) )
+        retcode = -1;
 
     if ( retcode == -1 )
-    {
-        wxLogError(_("Cannot execute program: %s"), cmdline.c_str());
-    }
+        throw Exception(wxString::Format(_("Cannot execute program: %s"), cmdline.c_str()));
 
     return retcode;
 }
 
+} // anonymous namespace
+
 
 bool ExecuteGettext(const wxString& cmdline)
 {
-    wxArrayString gstdout;
     wxArrayString gstderr;
-    int retcode = DoExecuteGettext(cmdline, gstdout, gstderr);
+    long retcode = DoExecuteGettext(cmdline, gstderr);
 
     for ( size_t i = 0; i < gstderr.size(); i++ )
     {
         if ( gstderr[i].empty() )
             continue;
-        wxLogError(_T("%s"), gstderr[i].c_str());
+        wxLogError("%s", gstderr[i].c_str());
     }
 
     return retcode == 0;
@@ -106,16 +157,15 @@ bool ExecuteGettext(const wxString& cmdline)
 
 bool ExecuteGettextAndParseOutput(const wxString& cmdline, GettextErrors& errors)
 {
-    wxArrayString gstdout;
     wxArrayString gstderr;
-    int retcode = DoExecuteGettext(cmdline, gstdout, gstderr);
+    long retcode = DoExecuteGettext(cmdline, gstderr);
 
-    wxRegEx reError(_T(".*\\.po:([0-9]+)(:[0-9]+)?: (.*)"));
+    wxRegEx reError(".*\\.po:([0-9]+)(:[0-9]+)?: (.*)");
 
     for ( size_t i = 0; i < gstderr.size(); i++ )
     {
         const wxString e = gstderr[i];
-        wxLogTrace(_T("poedit.execute"), _T("  stderr: %s"), e.c_str());
+        wxLogTrace("poedit.execute", "  stderr: %s", e.c_str());
         if ( e.empty() )
             continue;
 
@@ -128,16 +178,26 @@ bool ExecuteGettextAndParseOutput(const wxString& cmdline, GettextErrors& errors
             rec.line = (int)num;
             rec.text = reError.GetMatch(e, 3);
             errors.push_back(rec);
-            wxLogTrace(_T("poedit.execute"),
+            wxLogTrace("poedit.execute",
                        _T("        => parsed error = \"%s\" at %d"),
                        rec.text.c_str(), rec.line);
         }
         else
         {
-            wxLogTrace(_T("poedit.execute"), _T("        (unrecognized line!)"));
+            wxLogTrace("poedit.execute", "        (unrecognized line!)");
             // FIXME: handle the rest of output gracefully too
         }
     }
 
     return retcode == 0;
 }
+
+
+wxString QuoteCmdlineArg(const wxString& s)
+{
+    wxString s2(s);
+#ifdef __UNIX__
+    s2.Replace("\"", "\\\"");
+#endif
+    return "\"" + s2 + "\"";
+}
