@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2004-2013 Vaclav Slavik
+ *  Copyright (C) 2004-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -30,6 +30,8 @@
 #include "icons.h"
 #include "edapp.h"
 
+#ifndef __WXMAC__
+
 #ifdef __WXGTK20__
 // translates poedit item id or Tango stock item id to "legacy" GNOME id:
 static wxString GetGnomeStockId(const wxString& id)
@@ -55,18 +57,22 @@ wxBitmap PoeditArtProvider::CreateBitmap(const wxArtID& id,
                                          const wxArtClient& client,
                                          const wxSize& size)
 {
+    // Silence warning about unused parameter in some of the builds
+    (void)client;
+    (void)size;
+
     // Note: On Unix, this code is only called as last resolt, if standard
     //       theme provider (that uses current icon theme and files from
     //       /usr/share/icons/<theme>) didn't find any matching icon.
 
-    wxLogTrace(_T("poedit.icons"), _T("getting icon '%s'"), id.c_str());
+    wxLogTrace("poedit.icons", "getting icon '%s'", id.c_str());
 
 #ifdef __WXGTK20__
     // try legacy GNOME icons from standard theme:
     wxString gnomeId = GetGnomeStockId(id);
     if ( !gnomeId.empty() )
     {
-        wxLogTrace(_T("poedit.icons"), _T("-> legacy '%s'"), gnomeId.c_str());
+        wxLogTrace("poedit.icons", "-> legacy '%s'", gnomeId.c_str());
         wxBitmap gbmp(wxArtProvider::GetBitmap(gnomeId, client, size));
         if ( gbmp.Ok() )
             return gbmp;
@@ -74,24 +80,26 @@ wxBitmap PoeditArtProvider::CreateBitmap(const wxArtID& id,
 #endif // __WXGTK20__
 
     wxString iconsdir =
-#ifdef __WXMAC__
-        wxStandardPaths::Get().GetResourcesDir() + _T("/icons");
+#if defined(__WXMSW__)
+        wxStandardPaths::Get().GetResourcesDir() + "\\Resources";
 #else
-        wxGetApp().GetAppPath() + _T("/share/poedit/icons");
+        wxStandardPaths::Get().GetInstallPrefix() + "/share/poedit/icons";
 #endif
     if ( !wxDirExists(iconsdir) )
     {
-        wxLogTrace(_T("poedit.icons"),
-                   _T("icons dir %s not found"), iconsdir.c_str());
+        wxLogTrace("poedit.icons",
+                   "icons dir %s not found", iconsdir.c_str());
         return wxNullBitmap;
     }
 
     wxString icon;
-    icon.Printf(_T("%s/%s.png"), iconsdir.c_str(), id.c_str());
+    icon.Printf("%s/%s.png", iconsdir.c_str(), id.c_str());
     if ( !wxFileExists(icon) )
         return wxNullBitmap;
 
-    wxLogTrace(_T("poedit.icons"), _T("loading from %s"), icon.c_str());
+    wxLogTrace("poedit.icons", "loading from %s", icon.c_str());
     wxBitmap bmp(wxImage(icon, wxBITMAP_TYPE_ANY));
     return bmp;
 }
+
+#endif // !__WXMAC__
