@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2004-2013 Vaclav Slavik
+ *  Copyright (C) 2004-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -32,6 +32,7 @@
     #define HAS_THEMES_SUPPORT
 #endif
 
+#ifndef __WXMAC__
 class PoeditArtProvider : public wxArtProvider
 {
 protected:
@@ -39,5 +40,6 @@ protected:
                                   const wxArtClient& client,
                                   const wxSize& size);
 };
+#endif
 
 #endif // _ICONS_H_
