@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2001-2013 Vaclav Slavik
+ *  Copyright (C) 2001-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -29,10 +29,11 @@
 #include <wx/listctrl.h>
 #include <wx/xrc/xmlres.h>
 #include <wx/settings.h>
+#include <wx/stdpaths.h>
 #include <wx/intl.h>
 #include <wx/frame.h>
 #include <wx/listbox.h>
-#include "wxeditablelistbox.h"
+#include <wx/editlbox.h>
 #include <wx/tokenzr.h>
 #include <wx/msgdlg.h>
 #include <wx/button.h>
@@ -42,6 +43,8 @@
 #include <wx/log.h>
 #include <wx/artprov.h>
 #include <wx/iconbndl.h>
+#include <wx/windowptr.h>
+#include <wx/splitter.h>
 
 #include "catalog.h"
 #include "edapp.h"
@@ -67,37 +70,38 @@ ManagerFrame::ManagerFrame() :
     wxFrame(NULL, -1, _("Poedit - Catalogs manager"),
             wxDefaultPosition, wxDefaultSize,
             wxDEFAULT_FRAME_STYLE | wxNO_FULL_REPAINT_ON_RESIZE,
-            _T("manager"))
+            "manager")
 {
 #if defined(__WXGTK__)
     wxIconBundle appicons;
-    appicons.AddIcon(wxArtProvider::GetIcon(_T("poedit"), wxART_FRAME_ICON, wxSize(16,16)));
-    appicons.AddIcon(wxArtProvider::GetIcon(_T("poedit"), wxART_FRAME_ICON, wxSize(32,32)));
-    appicons.AddIcon(wxArtProvider::GetIcon(_T("poedit"), wxART_FRAME_ICON, wxSize(48,48)));
+    appicons.AddIcon(wxArtProvider::GetIcon("poedit", wxART_FRAME_ICON, wxSize(16,16)));
+    appicons.AddIcon(wxArtProvider::GetIcon("poedit", wxART_FRAME_ICON, wxSize(32,32)));
+    appicons.AddIcon(wxArtProvider::GetIcon("poedit", wxART_FRAME_ICON, wxSize(48,48)));
     SetIcons(appicons);
 #elif defined(__WXMSW__)
-    SetIcon(wxICON(appicon));
+    SetIcons(wxIconBundle(wxStandardPaths::Get().GetResourcesDir() + "\\Resources\\Poedit.ico"));
 #endif
 
     ms_instance = this;
 
-    wxXmlResource::Get()->LoadToolBar(this, _T("manager_toolbar"));
-    wxXmlResource::Get()->LoadMenuBar(this, _T("manager_menu"));
+    wxXmlResource::Get()->LoadToolBar(this, "manager_toolbar");
+    wxXmlResource::Get()->LoadMenuBar(this, "manager_menu");
 
-    wxPanel *panel = wxXmlResource::Get()->LoadPanel(this, _T("manager_panel"));
+    wxPanel *panel = wxXmlResource::Get()->LoadPanel(this, "manager_panel");
 
     m_listPrj = XRCCTRL(*panel, "prj_list", wxListBox);
     m_listCat = XRCCTRL(*panel, "prj_files", wxListCtrl);
+    m_splitter = XRCCTRL(*panel, "manager_splitter", wxSplitterWindow);
 
     wxImageList *list = new wxImageList(16, 16);
-    list->Add(wxArtProvider::GetBitmap(_T("poedit-status-cat-no")));
-    list->Add(wxArtProvider::GetBitmap(_T("poedit-status-cat-mid")));
-    list->Add(wxArtProvider::GetBitmap(_T("poedit-status-cat-ok")));
+    list->Add(wxArtProvider::GetBitmap("poedit-status-cat-no"));
+    list->Add(wxArtProvider::GetBitmap("poedit-status-cat-mid"));
+    list->Add(wxArtProvider::GetBitmap("poedit-status-cat-ok"));
     m_listCat->AssignImageList(list, wxIMAGE_LIST_SMALL);
 
     m_curPrj = -1;
 
-    int last = wxConfig::Get()->Read(_T("manager_last_selected"), (long)0);
+    int last = (int)wxConfig::Get()->Read("manager_last_selected", (long)0);
 
     // FIXME: do this in background (here and elsewhere)
     UpdateListPrj(last);
@@ -105,6 +109,8 @@ ManagerFrame::ManagerFrame() :
         UpdateListCat(last);
 
     RestoreWindowState(this, wxSize(400, 300));
+
+    m_splitter->SetSashPosition((int)wxConfig::Get()->Read("manager_splitter", 200));
 }
 
 
@@ -118,10 +124,12 @@ ManagerFrame::~ManagerFrame()
     int sel = m_listPrj->GetSelection();
     if (sel != -1)
     {
-        cfg->Write(_T("manager_last_selected"),
+        cfg->Write("manager_last_selected",
                    (long)m_listPrj->GetClientData(sel));
     }
 
+    wxConfig::Get()->Write("manager_splitter", (long)m_splitter->GetSashPosition());
+
     ms_instance = NULL;
 }
 
@@ -129,22 +137,23 @@ ManagerFrame::~ManagerFrame()
 void ManagerFrame::UpdateListPrj(int select)
 {
     wxConfigBase *cfg = wxConfig::Get();
-    int max = cfg->Read(_T("Manager/max_project_num"), (long)0) + 1;
+    long max = cfg->Read("Manager/max_project_num", (long)0) + 1;
     wxString key, name;
 
     m_listPrj->Clear();
     int item = 0;
     for (int i = 0; i <= max; i++)
     {
-        key.Printf(_T("Manager/project_%i/Name"), i);
+        key.Printf("Manager/project_%i/Name", i);
         name = cfg->Read(key, wxEmptyString);
         if (!name.empty())
         {
-            m_listPrj->Append(name, (void*)i);
+            m_listPrj->Append(name, (void*)(wxIntPtr)i);
             if (i == select)
             {
                 m_listPrj->SetSelection(item);
                 m_curPrj = select;
+                UpdateListCat(m_curPrj);
                 select = -1;
             }
             item++;
@@ -161,20 +170,20 @@ static void AddCatalogToList(wxListCtrl *list, int i, int id, const wxString& fi
     time_t modtime;
     wxString key;
     wxString file2(file);
-    file2.Replace(_T("/"), _T("_"));
-    file2.Replace(_T("\\"), _T("_"));
+    file2.Replace("/", "_");
+    file2.Replace("\\", "_");
     // FIXME: move cache to cache file and out of *config* file!
-    key.Printf(_T("Manager/project_%i/FilesCache/%s/"), id, file2.c_str());
+    key.Printf("Manager/project_%i/FilesCache/%s/", id, file2.c_str());
 
-    modtime = cfg->Read(key + _T("timestamp"), (long)0);
+    modtime = cfg->Read(key + "timestamp", (long)0);
 
     if (modtime == wxFileModificationTime(file))
     {
-        all = cfg->Read(key + _T("all"), (long)0);
-        fuzzy = cfg->Read(key + _T("fuzzy"), (long)0);
-        badtokens = cfg->Read(key + _T("badtokens"), (long)0);
-        untranslated = cfg->Read(key + _T("untranslated"), (long)0);
-        lastmodified = cfg->Read(key + _T("lastmodified"), _T("?"));
+        all = (int)cfg->Read(key + "all", (long)0);
+        fuzzy = (int)cfg->Read(key + "fuzzy", (long)0);
+        badtokens = (int)cfg->Read(key + "badtokens", (long)0);
+        untranslated = (int)cfg->Read(key + "untranslated", (long)0);
+        lastmodified = cfg->Read(key + "lastmodified", "?");
     }
     else
     {
@@ -188,12 +197,12 @@ static void AddCatalogToList(wxListCtrl *list, int i, int id, const wxString& fi
         cat.GetStatistics(&all, &fuzzy, &badtokens, &untranslated, NULL);
         modtime = wxFileModificationTime(file);
         lastmodified = cat.Header().RevisionDate;
-        cfg->Write(key + _T("timestamp"), (long)modtime);
-        cfg->Write(key + _T("all"), (long)all);
-        cfg->Write(key + _T("fuzzy"), (long)fuzzy);
-        cfg->Write(key + _T("badtokens"), (long)badtokens);
-        cfg->Write(key + _T("untranslated"), (long)untranslated);
-        cfg->Write(key + _T("lastmodified"), lastmodified);
+        cfg->Write(key + "timestamp", (long)modtime);
+        cfg->Write(key + "all", (long)all);
+        cfg->Write(key + "fuzzy", (long)fuzzy);
+        cfg->Write(key + "badtokens", (long)badtokens);
+        cfg->Write(key + "untranslated", (long)untranslated);
+        cfg->Write(key + "lastmodified", lastmodified);
     }
 
     int icon;
@@ -205,13 +214,13 @@ static void AddCatalogToList(wxListCtrl *list, int i, int id, const wxString& fi
     // FIXME: don't put full filename there, remove common prefix (of all
     //        directories in project's settings)
     list->InsertItem(i, file, icon);
-    tmp.Printf(_T("%i"), all);
+    tmp.Printf("%i", all);
     list->SetItem(i, 1, tmp);
-    tmp.Printf(_T("%i"), untranslated);
+    tmp.Printf("%i", untranslated);
     list->SetItem(i, 2, tmp);
-    tmp.Printf(_T("%i"), fuzzy);
+    tmp.Printf("%i", fuzzy);
     list->SetItem(i, 3, tmp);
-    tmp.Printf(_T("%i"), badtokens);
+    tmp.Printf("%i", badtokens);
     list->SetItem(i, 4, tmp);
     list->SetItem(i, 5, lastmodified);
 }
@@ -224,15 +233,15 @@ void ManagerFrame::UpdateListCat(int id)
 
     wxConfigBase *cfg = wxConfig::Get();
     wxString key;
-    key.Printf(_T("Manager/project_%i/"), id);
+    key.Printf("Manager/project_%i/", id);
 
-    wxString dirs = cfg->Read(key + _T("Dirs"), wxEmptyString);
+    wxString dirs = cfg->Read(key + "Dirs", wxEmptyString);
     wxStringTokenizer tkn(dirs, wxPATH_SEP);
 
     m_catalogs.Clear();
     while (tkn.HasMoreTokens())
         wxDir::GetAllFiles(tkn.GetNextToken(), &m_catalogs,
-                           _T("*.po"), wxDIR_FILES | wxDIR_DIRS);
+                           "*.po", wxDIR_FILES | wxDIR_DIRS);
 
     m_catalogs.Sort();
 
@@ -248,7 +257,7 @@ void ManagerFrame::UpdateListCat(int id)
 
     // FIXME: this is time-consuming, it should be done in parallel on
     //        multi-core/SMP systems
-    for (size_t i = 0; i < m_catalogs.GetCount(); i++)
+    for (int i = 0; i < (int)m_catalogs.GetCount(); i++)
         AddCatalogToList(m_listCat, i, id, m_catalogs[i]);
 
     m_listCat->SetColumnWidth(0, wxLIST_AUTOSIZE);
@@ -286,49 +295,56 @@ void ProjectDlg::OnBrowse(wxCommandEvent&)
     }
 }
 
-bool ManagerFrame::EditProject(int id)
+template<typename TFunctor>
+void ManagerFrame::EditProject(int id, TFunctor completionHandler)
 {
     wxConfigBase *cfg = wxConfig::Get();
     wxString key;
-    key.Printf(_T("Manager/project_%i/"), id);
+    key.Printf("Manager/project_%i/", id);
 
-    ProjectDlg dlg;
-    wxXmlResource::Get()->LoadDialog(&dlg, this, _T("manager_prj_dlg"));
-    wxXmlResource::Get()->AttachUnknownControl(_T("prj_dirs"),
-                new wxEditableListBox(this, -1, _("Directories:")));
+    wxWindowPtr<ProjectDlg> dlg(new ProjectDlg);
+    wxXmlResource::Get()->LoadDialog(dlg.get(), this, "manager_prj_dlg");
+    wxEditableListBox *prj_dirs = new wxEditableListBox(dlg.get(), XRCID("prj_dirs"), _("Directories:"));
+    wxXmlResource::Get()->AttachUnknownControl("prj_dirs", prj_dirs);
 
-    XRCCTRL(dlg, "prj_name", wxTextCtrl)->SetValue(cfg->Read(key + _T("Name")));
+    XRCCTRL(*dlg, "prj_name", wxTextCtrl)->SetValue(cfg->Read(key + "Name"));
 
-    wxString dirs = cfg->Read(key + _T("Dirs"));
-    wxArrayString adirs;
-    wxStringTokenizer tkn(dirs, wxPATH_SEP);
-    while (tkn.HasMoreTokens()) adirs.Add(tkn.GetNextToken());
-    XRCCTRL(*this, "prj_dirs", wxEditableListBox)->SetStrings(adirs);
-
-    if (dlg.ShowModal() == wxID_OK)
     {
-        cfg->Write(key + _T("Name"),
-                   XRCCTRL(dlg, "prj_name", wxTextCtrl)->GetValue());
-        XRCCTRL(*this, "prj_dirs", wxEditableListBox)->GetStrings(adirs);
-        if (adirs.GetCount() > 0)
-            dirs = adirs[0];
-        for (size_t i = 1; i < adirs.GetCount(); i++)
-            dirs << wxPATH_SEP << adirs[i];
-        cfg->Write(key + _T("Dirs"), dirs);
-
-        UpdateListPrj(id);
-        UpdateListCat(id);
-        return true;
+        wxString dirs = cfg->Read(key + "Dirs");
+        wxArrayString adirs;
+        wxStringTokenizer tkn(dirs, wxPATH_SEP);
+        while (tkn.HasMoreTokens())
+            adirs.Add(tkn.GetNextToken());
+        prj_dirs->SetStrings(adirs);
     }
-    else
-        return false;
+
+    dlg->ShowWindowModalThenDo([=](int retcode){
+        if (retcode == wxID_OK)
+        {
+            wxString dirs;
+            wxArrayString adirs;
+
+            cfg->Write(key + "Name",
+                       XRCCTRL(*dlg, "prj_name", wxTextCtrl)->GetValue());
+            prj_dirs->GetStrings(adirs);
+            if (adirs.GetCount() > 0)
+                dirs = adirs[0];
+            for (size_t i = 1; i < adirs.GetCount(); i++)
+                dirs << wxPATH_SEP << adirs[i];
+            cfg->Write(key + "Dirs", dirs);
+
+            UpdateListPrj(id);
+            UpdateListCat(id);
+        }
+        completionHandler(retcode == wxID_OK);
+    });
 }
 
 void ManagerFrame::DeleteProject(int id)
 {
     wxString key;
 
-    key.Printf(_T("Manager/project_%i"), id);
+    key.Printf("Manager/project_%i", id);
     wxConfig::Get()->DeleteGroup(key);
     UpdateListPrj();
 
@@ -363,25 +379,27 @@ END_EVENT_TABLE()
 void ManagerFrame::OnNewProject(wxCommandEvent&)
 {
     wxConfigBase *cfg = wxConfig::Get();
-    int max = cfg->Read(_T("Manager/max_project_num"), (long)0) + 1;
+    long max = cfg->Read("Manager/max_project_num", (long)0) + 1;
     wxString key;
     for (int i = 0; i <= max; i++)
     {
-        key.Printf(_T("Manager/project_%i/Name"), i);
+        key.Printf("Manager/project_%i/Name", i);
         if (cfg->Read(key, wxEmptyString).empty())
         {
-            m_listPrj->Append(_("<unnamed>"), (void*)i);
+            m_listPrj->Append(_("<unnamed>"), (void*)(wxIntPtr)i);
             m_curPrj = i;
-            if (EditProject(i))
-            {
-                if (i == max)
-                    cfg->Write(_T("Manager/max_project_num"), (long)max);
-            }
-            else
-            {
-                DeleteProject(i);
-            }
-            break;
+            EditProject(i, [=](bool added){
+                if (added)
+                {
+                    if (i == max)
+                        cfg->Write("Manager/max_project_num", (long)max);
+                }
+                else
+                {
+                    DeleteProject(i);
+                }
+            });
+            return;
         }
     }
 }
@@ -391,7 +409,7 @@ void ManagerFrame::OnEditProject(wxCommandEvent&)
 {
     int sel = m_listPrj->GetSelection();
     if (sel == -1) return;
-    EditProject((long)m_listPrj->GetClientData(sel));
+    EditProject((int)(wxIntPtr)m_listPrj->GetClientData(sel), [](bool){});
 }
 
 
@@ -401,7 +419,7 @@ void ManagerFrame::OnDeleteProject(wxCommandEvent&)
     if (sel == -1) return;
     if (wxMessageBox(_("Do you want to delete the project?"),
                _("Confirmation"), wxYES_NO | wxICON_QUESTION, this) == wxYES)
-        DeleteProject((long)m_listPrj->GetClientData(sel));
+        DeleteProject((int)(wxIntPtr)m_listPrj->GetClientData(sel));
 }
 
 
@@ -409,7 +427,7 @@ void ManagerFrame::OnSelectProject(wxCommandEvent&)
 {
     int sel = m_listPrj->GetSelection();
     if (sel == -1) return;
-    m_curPrj = (long)m_listPrj->GetClientData(sel);
+    m_curPrj = (int)(wxIntPtr)m_listPrj->GetClientData(sel);
     UpdateListCat(m_curPrj);
 }
 
@@ -439,9 +457,10 @@ void ManagerFrame::OnUpdateProject(wxCommandEvent&)
             else
             {
                 Catalog cat(f);
-                cat.Update(&pinfo);
+                cat.Update(&pinfo, /*summary=*/false, nullptr);
                 int validation_errors = 0;
-                cat.Save(f, false, validation_errors);
+                Catalog::CompilationStatus mo_status;
+                cat.Save(f, false, validation_errors, mo_status);
             }
          }
 
@@ -452,7 +471,9 @@ void ManagerFrame::OnUpdateProject(wxCommandEvent&)
 
 void ManagerFrame::OnOpenCatalog(wxListEvent& event)
 {
-    PoeditFrame::Create(m_catalogs[event.GetIndex()])->Raise();
+    PoeditFrame *f = PoeditFrame::Create(m_catalogs[event.GetIndex()]);
+    if (f)
+        f->Raise();
 }
 
 
