@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2001-2013 Vaclav Slavik
+ *  Copyright (C) 2001-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -31,6 +31,7 @@
 
 class WXDLLIMPEXP_FWD_CORE wxListCtrl;
 class WXDLLIMPEXP_FWD_CORE wxListBox;
+class WXDLLIMPEXP_FWD_CORE wxSplitterWindow;
 
 class Catalog;
 
@@ -60,7 +61,8 @@ class ManagerFrame : public wxFrame
         /** Pops up project settings dialog for project #id.
             \return false if user pressed Cancel, true otherwise
          */
-        bool EditProject(int id);
+        template<typename TFunctor>
+        void EditProject(int id, TFunctor completionHandler);
         /// Deletes project
         void DeleteProject(int id);
         /** Updates projects list
@@ -80,7 +82,8 @@ class ManagerFrame : public wxFrame
         void OnCloseCmd(wxCommandEvent& event);
 
         wxListCtrl *m_listCat;
-        wxListBox  *m_listPrj;      
+        wxListBox  *m_listPrj;
+        wxSplitterWindow *m_splitter;
         wxArrayString m_catalogs;
         int m_curPrj;
 
