@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2000-2013 Vaclav Slavik
+ *  Copyright (C) 2000-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -30,30 +30,45 @@
 #include <wx/notebook.h>
 
 #include "catalog.h"
+#include "languagectrl.h"
 
 class WXDLLIMPEXP_FWD_ADV wxEditableListBox;
 class WXDLLIMPEXP_FWD_CORE wxTextCtrl;
+class WXDLLIMPEXP_FWD_CORE wxRadioButton;
 class WXDLLIMPEXP_FWD_CORE wxComboBox;
 
 /// Dialog setting various catalog parameters.
 class PropertiesDialog : public wxDialog
 {
     public:
-        PropertiesDialog(wxWindow *parent = NULL);
+        PropertiesDialog(wxWindow *parent, bool fileExistsOnDisk, int initialPage = 0);
 
         /// Reads data from the catalog and fill dialog's controls.
         void TransferTo(Catalog *cat);
 
         /// Saves data from the dialog to the catalog.
         void TransferFrom(Catalog *cat);
+
+        virtual bool Validate();
             
     private:
+        void DisableSourcesControls();
+
+        void OnLanguageChanged(wxCommandEvent& event);
+        void OnLanguageValueChanged(const wxString& langstr);
+        void OnPluralFormsDefault(wxCommandEvent& event);
+        void OnPluralFormsCustom(wxCommandEvent& event);
+
         wxTextCtrl *m_team, *m_teamEmail, *m_project;
-        wxTextCtrl *m_language;
+        LanguageCtrl *m_language;
         wxComboBox *m_charset, *m_sourceCodeCharset;
-        wxTextCtrl *m_pluralForms;
+        wxRadioButton *m_pluralFormsDefault, *m_pluralFormsCustom;
+        wxTextCtrl *m_pluralFormsExpr;
         wxTextCtrl *m_basePath;
-        wxEditableListBox *m_paths, *m_keywords;
+        wxEditableListBox *m_paths, *m_excludedPaths, *m_keywords;
+        wxString m_rememberedPluralForm;
+
+        int m_validatedPlural, m_validatedLang;
 };
 
 
