@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2000-2013 Vaclav Slavik
+ *  Copyright (C) 2000-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -35,7 +35,7 @@
 
 MergeSummaryDialog::MergeSummaryDialog(wxWindow *parent)
 {
-    wxXmlResource::Get()->LoadDialog(this, parent, _T("summary"));
+    wxXmlResource::Get()->LoadDialog(this, parent, "summary");
 
     RestoreWindowState(this, wxDefaultSize, WinState_Size);
     CentreOnParent();
@@ -54,7 +54,7 @@ void MergeSummaryDialog::TransferTo(const wxArrayString& snew, const wxArrayStri
 {
     wxString sum;
     sum.Printf(_("(%i new, %i obsolete)"), 
-               snew.GetCount(), sobsolete.GetCount());
+               (int)snew.GetCount(), (int)sobsolete.GetCount());
     XRCCTRL(*this, "items_count", wxStaticText)->SetLabel(sum);
 
     wxListBox *listbox;
