@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2010-2013 Vaclav Slavik
+ *  Copyright (C) 2010-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -31,12 +31,16 @@
 #include <wx/config.h>
 #include <wx/display.h>
 
+#ifdef __WXOSX__
+#include <wx/cocoa/string.h>
+#endif
+
 wxString EscapeMarkup(const wxString& str)
 {
     wxString s(str);
-    s.Replace(_T("&"), _T("&amp;"));
-    s.Replace(_T("<"), _T("&lt;"));
-    s.Replace(_T(">"), _T("&gt;"));
+    s.Replace("&", "&amp;");
+    s.Replace("<", "&lt;");
+    s.Replace(">", "&gt;");
     return s;
 }
 
@@ -50,7 +54,7 @@ TempDirectory::TempDirectory() : m_counter(0)
 {
 #ifdef HAVE_MKDTEMP
     wxString path = wxFileName::GetTempDir();
-    path += _T("/poeditXXXXXX");
+    path += "/poeditXXXXXX";
     wxCharBuffer buf(path.fn_str());
     if ( mkdtemp(buf.data()) == NULL )
     {
@@ -61,7 +65,7 @@ TempDirectory::TempDirectory() : m_counter(0)
 #else
     for ( ;; )
     {
-        wxString name = wxFileName::CreateTempFileName(_T("poedit"));
+        wxString name = wxFileName::CreateTempFileName("poedit");
         if ( name.empty() )
         {
             wxLogError(_("Cannot create temporary directory."));
@@ -71,8 +75,14 @@ TempDirectory::TempDirectory() : m_counter(0)
         wxLogNull null;
         if ( wxRemoveFile(name) && wxMkdir(name, 0700) )
         {
+#ifdef __WXMSW__
+            // prevent possible problems with Unicode filenames in launched
+            // third party tools (e.g. gettext):
+            m_dir = wxFileName(name).GetShortPath();
+#else
             m_dir = name;
-            wxLogTrace(_T("poedit.tmp"), _T("created temp dir %s"), name.c_str());
+#endif
+            wxLogTrace("poedit.tmp", "created temp dir %s", name.c_str());
             break;
         }
         // else: try again
@@ -88,7 +98,7 @@ TempDirectory::~TempDirectory()
 
     if ( ms_keepFiles )
     {
-        wxLogTrace(_T("poedit.tmp"), _T("keeping temp files in %s"), m_dir.c_str());
+        wxLogTrace("poedit.tmp", "keeping temp files in %s", m_dir.c_str());
         return;
     }
 
@@ -96,28 +106,74 @@ TempDirectory::~TempDirectory()
     {
         if ( wxFileName::FileExists(*i) )
         {
-            wxLogTrace(_T("poedit.tmp"), _T("removing temp file %s"), i->c_str());
+            wxLogTrace("poedit.tmp", "removing temp file %s", i->c_str());
             wxRemoveFile(*i);
         }
     }
 
-    wxLogTrace(_T("poedit.tmp"), _T("removing temp dir %s"), m_dir.c_str());
+    wxLogTrace("poedit.tmp", "removing temp dir %s", m_dir.c_str());
     wxFileName::Rmdir(m_dir);
 }
 
 
 wxString TempDirectory::CreateFileName(const wxString& suffix)
 {
-    wxString s = wxString::Format(_T("%s%c%d%s"),
+    wxString s = wxString::Format("%s%c%d%s",
                                   m_dir.c_str(), wxFILE_SEP_PATH,
                                   m_counter++,
                                   suffix.c_str());
     m_files.push_back(s);
-    wxLogTrace(_T("poedit.tmp"), _T("new temp file %s"), s.c_str());
+    wxLogTrace("poedit.tmp", "new temp file %s", s.c_str());
     return s;
 }
 
 
+// ----------------------------------------------------------------------
+// TempOutputFile
+// ----------------------------------------------------------------------
+
+TempOutputFileFor::TempOutputFileFor(const wxString& filename)
+{
+    wxString path, name, ext;
+    wxFileName::SplitPath(filename, &path, &name, &ext);
+
+#ifdef __WXOSX__
+    NSURL *fileUrl = [NSURL fileURLWithPath:[NSString stringWithUTF8String: filename.utf8_str()]];
+    NSURL *tempdirUrl =
+        [[NSFileManager defaultManager] URLForDirectory:NSItemReplacementDirectory
+                                               inDomain:NSUserDomainMask
+                                      appropriateForURL:[fileUrl URLByDeletingLastPathComponent]
+                                                 create:YES
+                                                  error:nil];
+    if (tempdirUrl)
+    {
+        NSURL *newFileUrl = [tempdirUrl URLByAppendingPathComponent:[fileUrl lastPathComponent]];
+        NSString *newFilePath = [newFileUrl path];
+        m_filenameTmp = wxStringWithNSString(newFilePath);
+        m_tempDir = wxStringWithNSString([tempdirUrl path]);
+    }
+    // else: fall through to the generic code
+#endif // __WXOSX__
+
+    if (m_filenameTmp.empty())
+    {
+        m_filenameTmp = filename + ".temp." + ext;
+    }
+
+    if ( wxFileExists(m_filenameTmp) )
+        wxRemoveFile(m_filenameTmp);
+}
+
+
+TempOutputFileFor::~TempOutputFileFor()
+{
+#ifdef __WXOSX__
+    if (!m_tempDir.empty())
+        wxFileName::Rmdir(m_tempDir, wxPATH_RMDIR_RECURSIVE);
+#endif
+}
+
+
 // ----------------------------------------------------------------------
 // Helpers for persisting windows' state
 // ----------------------------------------------------------------------
@@ -131,24 +187,25 @@ void SaveWindowState(const wxTopLevelWindow *win, int flags)
     {
         if ( !win->IsMaximized() )
         {
-#ifdef __WXMSW__
+#if defined(__WXMSW__) || defined(__WXOSX__)
+
             if ( flags & WinState_Pos )
             {
                 const wxPoint pos = win->GetPosition();
-                cfg->Write(path + _T("x"), (long)pos.x);
-                cfg->Write(path + _T("y"), (long)pos.y);
+                cfg->Write(path + "x", (long)pos.x);
+                cfg->Write(path + "y", (long)pos.y);
             }
-#endif // __WXMSW__
+#endif // __WXMSW__/__WXOSX__
             if ( flags &  WinState_Size )
             {
                 const wxSize sz = win->GetClientSize();
-                cfg->Write(path + _T("w"), (long)sz.x);
-                cfg->Write(path + _T("h"), (long)sz.y);
+                cfg->Write(path + "w", (long)sz.x);
+                cfg->Write(path + "h", (long)sz.y);
             }
         }
 
         if ( flags &  WinState_Size )
-            cfg->Write(path + _T("maximized"), (long)win->IsMaximized());
+            cfg->Write(path + "maximized", (long)win->IsMaximized());
     }
 }
 
@@ -160,30 +217,62 @@ void RestoreWindowState(wxTopLevelWindow *win, const wxSize& defaultSize, int fl
 
     if ( flags & WinState_Size )
     {
-        int width = cfg->Read(path + _T("w"), defaultSize.x);
-        int height = cfg->Read(path + _T("h"), defaultSize.y);
+        int width = (int)cfg->Read(path + "w", defaultSize.x);
+        int height = (int)cfg->Read(path + "h", defaultSize.y);
         if ( width != -1 || height != -1 )
             win->SetClientSize(width, height);
     }
 
-#ifdef __WXMSW__
+#if defined(__WXMSW__) || defined(__WXOSX__)
     if ( flags & WinState_Pos )
     {
-        int posx = cfg->Read(path + _T("x"), -1);
-        int posy = cfg->Read(path + _T("y"), -1);
-        if ( posx != -1 || posy != -1 )
-            win->Move(posx, posy);
+        wxPoint pos;
+        pos.x = (int)cfg->Read(path + "x", -1);
+        pos.y = (int)cfg->Read(path + "y", -1);
+        if ( pos.x != -1 || pos.y != -1 )
+        {
+            // NB: if this is the only Poedit frame opened, place it at remembered
+            //     position, but don't do that if there already are other frames,
+            //     because they would overlap and nobody could recognize that there are
+            //     many of them
+            for (;;)
+            {
+                bool occupied = false;
+                for (auto& w : wxTopLevelWindows)
+                {
+                    if (w != win && w->GetPosition() == pos)
+                    {
+                        occupied = true;
+                        break;
+                    }
+                }
+                if (!occupied)
+                    break;
+                pos += wxPoint(20,20);
+            }
+
+            win->Move(pos);
+        }
     }
 
     // If the window is completely out of all screens (e.g. because
     // screens configuration changed), move it to primary screen:
+#ifdef __WXOSX__
+    if ( win->GetPosition().x < 0 || win->GetPosition().y < 0 )
+        win->Move(20, 30);
+#else
     if ( wxDisplay::GetFromWindow(win) == wxNOT_FOUND )
         win->Move(0, 0);
-#endif // __WXMSW__
+#endif
 
+#endif // __WXMSW__/__WXOSX__
+
+#ifndef __WXOSX__
     // If the window is larger than current screen, resize it to fit:
     int display = wxDisplay::GetFromWindow(win);
-    wxCHECK_RET( display != wxNOT_FOUND, _T("window not on screen") );
+    if ( display == wxNOT_FOUND )
+        return;
+
     wxRect screenRect = wxDisplay(display).GetClientArea();
 
     wxRect winRect = win->GetRect();
@@ -197,8 +286,9 @@ void RestoreWindowState(wxTopLevelWindow *win, const wxSize& defaultSize, int fl
     }
 
     // Maximize if it should be
-    if ( cfg->Read(path + _T("maximized"), long(0)) )
+    if ( cfg->Read(path + "maximized", long(0)) )
     {
         win->Maximize();
     }
+#endif // !__WXOSX__
 }
