@@ -1,7 +1,7 @@
 /*
- *  This file is part of Poedit (http://www.poedit.net)
+ *  This file is part of Poedit (http://poedit.net)
  *
- *  Copyright (C) 2010-2013 Vaclav Slavik
+ *  Copyright (C) 2010-2014 Vaclav Slavik
  *
  *  Permission is hereby granted, free of charge, to any person obtaining a
  *  copy of this software and associated documentation files (the "Software"),
@@ -26,6 +26,12 @@
 #ifndef _UTILITY_H_
 #define _UTILITY_H_
 
+#ifndef HAVE_MKDTEMP
+    #ifdef __WXOSX__
+        #define HAVE_MKDTEMP
+    #endif
+#endif
+
 #include <wx/string.h>
 #include <wx/arrstr.h>
 #include <wx/toplevel.h>
@@ -61,6 +67,24 @@ private:
     static bool ms_keepFiles;
 };
 
+// Holder of temporary file for creating the output.
+class TempOutputFileFor
+{
+public:
+    explicit TempOutputFileFor(const wxString& filename);
+    ~TempOutputFileFor();
+
+    const wxString& FileName() const { return m_filenameTmp; }
+
+#ifdef __WXOSX__
+    wxString m_tempDir;
+#endif
+    wxString m_filenameTmp;
+};
+
+// Helper for writing files
+
+
 // ----------------------------------------------------------------------
 // Helpers for persisting windows' state
 // ----------------------------------------------------------------------
@@ -78,7 +102,7 @@ void RestoreWindowState(wxTopLevelWindow *win, const wxSize& defaultSize,
 
 inline wxString WindowStatePath(const wxWindow *win)
 {
-    return wxString::Format(_T("/windows/%s/"), win->GetName().c_str());
+    return wxString::Format("/windows/%s/", win->GetName().c_str());
 }
 
 #endif // _UTILITY_H_
